# Tabsize 4 --> <= 80 Zeichen/Zeile


#Namensraum fr das Analysemodul

namespace eval Analyzer {

	global global_options

	# Analysefunktion exportieren
	namespace export analyze
	namespace export clear_db_result
	namespace export count_regex
	namespace export get_getDeltaValuePage
	namespace export get_options
	namespace export get_qst_number
	namespace export get_qst_text
	namespace export get_subquestion_ids
	namespace export is_empty_row
	namespace export is_mehrfachauswahl
	namespace export is_qst_type_auswahl
	namespace export is_qst_type_block
	namespace export is_qst_type_text


	# Plausibilittsprfung importieren
	source [file join "$global_options(BASEDIR)" "plausibel.tcl"]

	# Konstanten
	variable ANALYZER_DEBUGMODE 0 ;# 0=Debug aus, 1=Debug ein
	variable OPTION_REGEX {^([^=]*)=([^\n]*)\n?} ;# Regexp. fr die Optionen
	variable page_default_title "Auswertung" ;# Titel der Auswertungsseite
	variable ORDERNR_FRM_GENERAL_DATA 0 ;# Default-Ordernr., unter
	            # der die allg. FB-Daten gespeichert werden

	# Infix fr Antworten in der results
    # -Tabelle (z.B. bei qst3-2)
	variable ANSWER_INFIX "-"

    # Format fr Kommazahlen
    variable FLOAT_FMT "%0.2f"
}


########################################################################
###########################    Oeffentlich    ##########################
########################################################################


############### Funktionsbeschreibung ##################################
# Zweck: Hauptfunktion fr die Analyse eines Fragebogens. Ist deltavalue
#        leer, so wird zunchst eine xml-Seite zurckgegeben, die
#        die Eingabe von delta ermglicht
#
# Rckgabewert: Pseudo-xml-Ergebnis der Analyse
#
# Parameter: formid: ID des Fragebogens, password: Passwort,
#            deltavalue: Delta-Wert (s. Handbuch)
########################################################################
proc Analyzer::analyze {formid password deltavalue} {
	variable page_default_title

	global dbcErrorMessage ;# DB-Fehlermeldung
    	global global_options

	set res {} ;# Ergebnis-Initialisierung


	# Passwort berprfen
	set passwd_msg [Util::check_FormID_And_Password $formid $password]

	# wenn Password ok ist...
	if {$passwd_msg == {}} {


	# wenn Delta-Wert leer ist --> Formular ausgeben, Passwort
	# wird mitgefhrt
	if {$deltavalue == {}} {
	    return [get_getDeltaValuePage $formid $password]
	}


	# Plausibilittsprfung durchfhren mit Delta-Wert
	set plausibel_res\
		[Plausibel::plausibilitaetspruefung $formid $deltavalue]

	# Versuche Datenbank zu ffnen (Infos durch dbinfo.tcl, wurde
	# in generator.tcl gesourced
	set dbstatus [dbcopen]
	add_debug_msg res "ffne DB (1=OK)... $dbstatus"
	add_debug_msg res "DBC Errormessage ---$dbcErrorMessage---"

	# Wenn die DB geffnet werden konnte
	if {$dbstatus} {
	add_debug_msg res "Formular: $formid"
	add_debug_msg res "Delta-Wert: $deltavalue Sekunden"

	# Ermittle Anz. Fragen
	add_debug_msg res  "Ermittle Anzahl der Fragen..."


	append res "<plausibilitaets-section>\
		$plausibel_res\
		</plausibilitaets-section>"


	# WHERE-Klausel fr Selektion aller Fragen
	set where "WHERE ATTRIBUTE='QstNumber'\
		   AND FORMID='$formid'\
		   ORDER BY ORDERNO ASC"

	# Zhlen der Fragen, festhalten in qst_count
	clear_db_result
	set qst_count [dbcselect ORDERNO $global_options(TBL_FORMDATA) $where]
	add_debug_msg res  "Anz. Fragen: $qst_count"

	# Wenn es keine Fragen gibt, Fehlermeldung!
	if {[expr $qst_count < 1]} {
		add_err_msg res {Der Fragebogen enthlt keine Fragen zum Auswerten\
				oder er existiert nicht!}
	} else {
		# es sind Fragen vorhanden

		add_debug_msg res "Anz. Fragen ok"
		add_debug_msg res "Ermittle Liste der relevanten ORDERNO..."

		# die zugehrigen Order-Nummern aus der eben gettigten Abfrage
		# auslesen und in qst_order_numbers speichern
		set qst_order_numbers {}
		set row [dbcfetch]
		while {![is_empty_row $row]} {
			lappend qst_order_numbers [lindex $row 1]
			set row [dbcfetch]
		}

		add_debug_msg res "Liste der ORDERNO: $qst_order_numbers"
		add_debug_msg res "\n\n*** Beginn der eigtl. Auswertung ***"

		# allg. Fragebogendaten ermitteln als xml
		append res [get_general_frm_data $formid]


		# alle Order-Nummern in qst_order_numbers (also alle Fragen)
		# werden durchlaufen:

		foreach order_nr $qst_order_numbers {
			add_debug_msg res "\n\n\n\
			*******************************************************
			\nAuswertung ORDERNO $order_nr"

			# Auswertung-Sektion fr eine Frage beginnt
			append res "\n\n\n<auswertung-section>"
			append res "\n[get_qst_general_data $formid $order_nr]"

			# Fragetyp ermitteln und die richtige Analysefunktion
			# fr den jew. Fragetyp ermitteln:

			# Blockfrage:
			if {[is_qst_type_block $formid $order_nr]} {
				add_debug_msg res "Analysemodus: Blockfrage"
				append res [analyze_qst_type_block $formid $order_nr]
			} else {
			# Textfrage:
			if {[is_qst_type_text $formid $order_nr]} {
				add_debug_msg res "Analysemodus: Textfrage"
				append res [analyze_qst_type_text $formid $order_nr]
			} else {
			# Auswahlfrage:
			if {[is_qst_type_auswahl $formid $order_nr]} {
				add_debug_msg res "Analysemodus: Auswahlfrage"
				append res [analyze_qst_type_auswahl $formid $order_nr]
			} else {
			# unbekannter Fragetyp
				add_err_msg res {FRAGETYP UNBEKANNT - KEINE AUSWERTUNG}
			}}}
			# Ende der Fragenauswertung aktuelle Frage,
			# es folgt die nchste Frage
			append res "\n</auswertung-section>"
		}
	}

	# Die ungltig-Markierungen der Plausibilittsprfung mu wieder
	# rckgngig gemacht werden (wichtig, damit spter P.prfungen
	# mit anderen delta-Werten erfolgen knnen)
	Plausibel::undo_invalidation $formid

	dbcclose ;# DB schlieen
	} else {
		# Fehlermeldung: DB konnte nicht geffnet werden
		add_err_msg res "Konnte Datenbank nicht ffnen!"
	}
	} else {
		# Fehlermeldung: Passwort-Fehler
		return "$passwd_msg"
	}

	# Das resultat als xml-Seite zurckgeben
	return "<style style=\"analyze.style\"><page-defaultbegin\
			title=Auswertemodul>\
			<body-defaultnavigation>\
			<body-content>$res</body-content>\
            	<page-defaultend>"
}








########################################################################
###########################    Privat    ###############################
########################################################################


############### Funktionsbeschreibung ##################################
# Zweck: konvertiert eine Liste von absoluten Hufigkeiten in relative
#        Hufigkeiten (anhand der Anz. der Personen (n), die die jew.
#        Frage ausgefllt haben)
#
# Rckgabewert: Liste mit entsprechenden relativen Hufigkeiten
#
# Parameter: abs_werteliste: Liste mit absoluten Werten
#            formid: Fragebogen-ID
#            qst_number: Nummer der Frage (z.B. 'qst3-8' oder 'qst4')
#
########################################################################
proc Analyzer::absolute2relative {abs_werteliste formid qst_number} {
	set i 0
	set n [get_anzahl_ausgefuellt $formid $qst_number]
	set res {}
    variable FLOAT_FMT

	# abs. Hufigkeiten durchlaufen
	while {[expr $i < [llength $abs_werteliste]]} {
		# aktuelle abs. Hufigkeit durch n teilen (sichere
		# Division)
		lappend res [format $FLOAT_FMT [divide [lindex $abs_werteliste $i] $n]]

		incr i ;# nchste abs. Hufigkeit
	}

	#Liste zurckgeben
	return $res
}


############### Funktionsbeschreibung ##################################
# Zweck: fgt einem String einen Debug-Text mit xml-Tags hinzu, WENN
#        ANALYZER_DEBUGMODE > 0 ist
#
# Rckgabewert: -
#
# Parameter: varname: Variablenname, der der Debug-Text hinzugefgt 
#                     werden soll
#            msg: Die hinzuzufgende Debug-Meldung
#
########################################################################
proc Analyzer::add_debug_msg {varname msg} {
	variable ANALYZER_DEBUGMODE
	upvar $varname text ;# call by name

	# hinzufgen der Meldung, wenn Debug-Mode an ist
	if {$ANALYZER_DEBUGMODE} {
		append text "\n<analyzer-debug>$msg</analyzer-debug>"
	}
}


############### Funktionsbeschreibung ##################################
# Zweck: fgt einem String eine Fehlermeldung mit xml-Tags hinzu
#
# Rckgabewert: -
#
# Parameter: varname: Variablenname, der der Debug-Text hinzugefgt 
#                     werden soll
#            errmsg: Die hinzuzufgende Fehlermeldung
#
########################################################################
proc Analyzer::add_err_msg {varname errmsg} {
	upvar $varname text ;# call by name

	# Fehlermeldung anhngen:
	append text "<analyzer-error>$errmsg</analyzer-error>"
}


############### Funktionsbeschreibung ##################################
# Zweck: Analysiert eine Auswahlfrage
#
# Rckgabewert: xml-Code, der das Analyseergebnis enthlt
#
# Parameter: formid: Fragebogen-ID
#            order_nr: ordernr der zu analysierenden Frage
#
########################################################################
proc Analyzer::analyze_qst_type_auswahl {formid order_nr} {
	# Fragenummer herausfinden
	set qst_number [get_qst_number $formid $order_nr]
	set res {}


	# Liste der mglichen Optionen holen
	set options [get_options $formid $order_nr]
	
	add_debug_msg res "(analyze_qst_type_auswahl) QstNumber ist $qst_number"
	add_debug_msg res "(analyze_qst_type_auswahl) Optionen: $options"

	# Wenn die Fragenummer gefunden wurde
	if {$qst_number != {}} {

		#Antworten fr jede Antwortmglichkeit zhlen
		set stat_list [get_stat_list_for_auswahlfrage $formid $qst_number\
                        $options]

		add_debug_msg res "Statistische Liste: $stat_list"

		append res [get_statistic_info_for_auswahlfrage [lindex $stat_list 0]\
                    [lindex $stat_list 1] [lindex $stat_list 2]]
		append res [get_ausfuellstatus_section $formid $qst_number]

	} else {
		# Fragenummer nicht gefunden --> Fehler
		append res [get_err {Kann Fragenummer nicht finden!}]
	}

	# xml-Text zurckgeben
	return $res
}


############### Funktionsbeschreibung ##################################
# Zweck: Analysiert eine Blockfrage
#
# Rckgabewert: xml-Code, der das Analyseergebnis enthlt
#
# Parameter: formid: Fragebogen-ID
#            order_nr: ordernr der zu analysierenden Frage
#
########################################################################
proc Analyzer::analyze_qst_type_block {formid order_nr} {
	# Fragenummer ermitteln (Prfix), es gibt ja noch Unterfragen
	set qst_number [get_qst_number $formid $order_nr]

	# Default-Initialisierungen:
	set res {}
	set subquestion_id 0
	set n_subquestions 0

	# Liste aller mglichen Optionen holen
	set options [get_options $formid $order_nr]

	# Unterfragen ermitteln (Rohformat wie in DB)
	set subquestions [get_subquestions $formid $order_nr]

	# Unterfragen in Liste umwandeln
	set subquestions_as_list [get_subquestions_as_list $subquestions]

	# Anzahl der Unterfragen ermitteln
	set n_subquestions [count_rows $subquestions]

	# die Suffixes der Unterfagen-Nummern ermitteln
	set subquestions_ids [get_subquestion_ids $subquestions $qst_number]

	# *******************************************************************
	# Listen fr Listen mit abs./rel. Hufigkeiten fr alle Unterfragen:
	# {{h_0_0 h_0_1 h_0_2 ... _0_N} {h_1_0 h_1_1 h_1_2 ... h_1_N} ...
	# {h_k_0 h_k_1 h_k_2 ... h_k_N}}
	# N: Anzahl Optionen, k: Anzahl Unterfragen,
	# h_i_m: Hufigkeit Unterfrage i, Option m
	# *************************************************************
	# Liste fr Listen mit absoluten Hufigkeiten fr alle Unterfragen
	set hlists_absolute {}
	# Liste fr Listen mit relativen Hufigkeiten fr alle Unterfragen
	set hlists_relative {}
	# *******************************************************************

	# Liste fr die Ausfllstatus fr alle Unterfragen (als xml-Text)
	#   Format: {ausfuell0, ..., ausfuellK} mit k: Anzahl Unterfragen
	set ausfuellstatus_lists {}

	# Liste fr die berschriften (=Optionsbezeichner d. Auswahlfragen)
	#  Format: {option0, ..., optionN} mit N: Anzahl Optionen
	set headings {}

	add_debug_msg res "(analyze_qst_type_block) QstNumber ist $qst_number"
	add_debug_msg res "(analyze_qst_type_block) Optionen: $options"
	add_debug_msg res "subquestions_ids: ---$subquestions_ids---"

	# wenn die Nummer der Frage ermittelt werden konnte
	if {$qst_number != {}} {
	
		# Antworten fr jede Option zhlen und abs./rel. Hufigkeiten
		# bilden, alle Unterfragen-Nummern durchlaufen, z.B.
		# {qst3-0 qst3-1 qst3-2 usw.}
		foreach subquestion_id $subquestions_ids {

			# Ermittle eine Liste mit 3 Listen (Optionsbezeichner,
			# abs., rel. Hufigkeiten			
			set stat_list_auswahl [get_stat_list_for_auswahlfrage $formid\
                $subquestion_id $options]

			# Hole Liste mit absoluten Hufigkeiten
			set stat_list [lindex $stat_list_auswahl 1]

			# wenn Optionsbezeichner noch nicht gesetzt:
			# --> diese ermitteln, aber nur einmal!
			if {$headings == {}} {
				set headings [lindex $stat_list_auswahl 0]
			}

			# fr akt. Unterfrage Liste mit abs. Hufigkeiten
			# an die Gesamtliste fr die abs. Hufigkeiten anhngen
			lappend hlists_absolute $stat_list

			# fr akt. Unterfrage Liste mit rel. Hufigkeiten
			# an die Gesamtliste fr die rel. Hufigkeiten anhngen
			lappend hlists_relative\
			  [absolute2relative $stat_list $formid $subquestion_id]

			# Ausfllstatus fr akt. Unterfrage an Ausfllstatus-
			# liste anhngen
			lappend ausfuellstatus_lists\
			  [get_ausfuellstatus_section $formid $subquestion_id]
		}

		add_debug_msg res "Headings: $headings"
		add_debug_msg res "Unterfragen: $subquestions_as_list"
		add_debug_msg res "Statistische Liste: (absolute H.):\
		  $hlists_absolute"
		add_debug_msg res "Statistische Liste: (relative H.):\
		  $hlists_relative"
		add_debug_msg res "Ausfllstatus-Liste: $ausfuellstatus_lists"

		# aus den gewonnenen Informationen fr die Unterfragen wird nun
		# xml-Text generiert:
		append res [get_statistic_info_for_blockfrage $headings\
		  $subquestions_as_list $hlists_absolute $hlists_relative\
		  $ausfuellstatus_lists]

	} else {
		append res [get_err {Kann Fragenummer n. ermitteln}]
	}	
	return $res
}


############### Funktionsbeschreibung ##################################
# Zweck: Analysiert eine Textfrage
#
# Rckgabewert: xml-Code, der das Analyseergebnis enthlt (alle Ant-
#		worten
#
# Parameter: formid: Fragebogen-ID
#            order_nr: ordernr der zu analysierenden Frage
#
########################################################################
proc Analyzer::analyze_qst_type_text {formid order_nr} {
    global global_options

	# Fragenr. ermitteln
	set qst_number [get_qst_number $formid $order_nr]

	set res {}
	add_debug_msg res "(analyze_qst_type_text) QstNumber ist $qst_number"


	# alle Antworten selektieren, NULL-Antworten ignorieren
	set where "WHERE FORMID='$formid' AND QUESTIONNO='$qst_number' AND\
                   ANSWER IS NOT NULL ORDER BY SESSIONID ASC"
	clear_db_result

	dbcselect {ANSWER} $global_options(TBL_RESULTS) $where

	# wenn Fragenummer ermittelt werden konnte...
	if {$qst_number != {}} {

		# durchlaufe alle Textantworten
		set row [dbcfetch]
		set res "$res\n<textanswer-section>"

		while {![is_empty_row $row]} {
			# f. jede Antwort xml-Code generieren
			set res "$res\n\t<textanswer>\
			  [lindex [lindex $row 1] 0]</textanswer>"

			# nchste Antwort
			set row [dbcfetch]
		}

		# Ende der Auflistung d. Antworten	
		append res "\n</textanswer-section>"

		# Ausfllstatus ermitteln als xml und anhngen an Ergebnis
		append res [get_ausfuellstatus_section $formid $qst_number]
	} else {
		# wenn Fragenummer nicht ermittelt werden konnte --> Fehler
		append res [get_err {Kann Fragenr. f. Textfrage n. ermitteln!}]
	}	

	return $res
}



############### Funktionsbeschreibung ##################################
# Zweck: Prft, ob in Formdata ein Attribut mit dem Namen attr_name
#        (Spalte Attribute) existiert (fr eine bestimmte Frage)
#
# Rckgabewert: 1 Attribut existiert
#               0 sonst
#
# Parameter: formid: Fragebogen-ID
#            order_nr: Ordernr. der Frage
#            attr_name: zu suchender Attributname
#
########################################################################
proc Analyzer::attr_exists {formid order_nr attr_name} {
    global global_options

	# suche nach dem Attributnamen:
	set where "WHERE ATTRIBUTE='$attr_name' AND ORDERNO=$order_nr\
             	   AND FORMID='$formid'"
	clear_db_result
	set res "[dbcselect orderno $global_options(TBL_FORMDATA) $where]"

	# > 0 Tupel gefunden?	
	return [expr $res > 0]
}


############### Funktionsbeschreibung ##################################
# Zweck: entfernt alle momentan selektierten Datenstze aus dem Puffer,
#        dies ist z.B. sinnvoll, wenn die Datenstze einer vorherigen
#	 Abfrage nicht vollstndig ausgelesen wurden, weil sie nicht
#	 mehr von Interese sind
#
# Rckgabewert: -
#
# Parameter: -
#
########################################################################
proc Analyzer::clear_db_result {} {
	# alle Zeilen auslesen und vergessen:
	set row [dbcfetch]
	while {![is_empty_row $row]} {
		set row [dbcfetch]
	}
}


############### Funktionsbeschreibung ##################################
# Zweck: zhlt das Vorkommen eines regulren Ausdruckes regex im String
#	 s
#
# Rckgabewert: Anzahl der Vorkommen von regex in s
#
# Parameter: regex: regulrer Ausdruck
#	     s: String
#
########################################################################
proc Analyzer::count_regex {regex s} {
	set res 0
	set match {}

	# suche regex und ersetze alle Vorkommen:
	while {[regexp $regex $s match]} {
		regsub $regex $s {} s
		# fr jedes Vorkommen Resultat um 1 erhhen
		incr res
	}

	return $res
}


############### Funktionsbeschreibung ##################################
# Zweck: zhlt die Zeilen in einem String s (eine Zeile wird durch
#	 \n abgeschlossen
#
# Rckgabewert: Anzahl der Zeilen in s
#
# Parameter: String s, der die Zeilen als Text enthlt, durch \n
#	     getrennt
#
########################################################################
proc Analyzer::count_rows {s} {
	# mit regulrem Ausdruck Zeilen zhlen:
	return [expr 1 + [count_regex "^\[^\n\]*\n" $s]]
}


############### Funktionsbeschreibung ##################################
# Zweck: erstellt aus einer Liste aus Listen mit xml-Elementen einen
#        xml-Text, der die Listenstruktur auflst und z.B. zum Erstellen
#	 einer HTML-Tabelle/Liste benutzt werden kann.
#	 Die Liste {{a b c} {d e f} {g h i j}} wird z.B. zu
#	 <statistic-info-section>
#	  <statistic-row>abc</statistic-row>
#	  <statistic-row>def</statistic-row>
#	  <statistic-row>ghij</statistic-row>
#	 </statistic-info-section>
#
# Rckgabewert: o.g. xml-Text
#
# Parameter: stat_list: Liste der Form
#		{{e_0_0... e_0_N0} {e_1_0...e_1_N1}...}
#		mit e_i_k: Elemente k der Liste i,
#		    Ni (i=0..m) Anzahl der Elemente der
#		    Liste i, es gibt in stat_list m Listen
########################################################################
proc Analyzer::create_statistic_xml_info_from_list {stat_list} {
	set res {}

	# jede Zeile in Statistic Row-Tag einbetten
	foreach sublist $stat_list {
		append res "<statistic-row>"

		# die Zellen schreiben
		foreach element $sublist {
			append res " $element"
		}
		append res "</statistic-row>\n" ;# Ende der Zeile
	}

	# das ganze ist ein statistischer Abschnitt
	return "\n<statistic-section>\n$res</statistic-section>"
}


############### Funktionsbeschreibung ##################################
# Zweck: sicheres Dividieren
#
# Rckgabewert: Quotient aus a und b, bei Division durch 0 '-'
#		
# Parameter: a: Dividend, b: Divisor
#
########################################################################
proc Analyzer::divide {a b} {
	if {[expr $b != 0]} {
		return [expr [expr double($a) / double($b)] * 100]
	} else {
        if {[expr $a == 0]} {
            return 0
        } else {return "?"}
	}
}


############### Funktionsbeschreibung ##################################
# Zweck: gibt auf stderr eine Debug-Meldung aus, wenn der Debug-Modus
#	 eingeschaltet ist (ANALYZER_DEBUGMODE 1)
#
# Rckgabewert: -
#
# Parameter: Die Fehlermeldung
#
########################################################################
proc Analyzer::dputs {msg} {
	variable ANALYZER_DEBUGMODE

	if {$ANALYZER_DEBUGMODE} {
		puts stderr "!DEBUG! $msg"
	}	
}


############### Funktionsbeschreibung ##################################
# Zweck: Berechnet die Anzahl der Datenstze, die durch ein
#	 bestimmtes SQL-Statement selektiert werden
#
# Rckgabewert: Anzahl der Datenstze, die selektiert wurden
#
# Parameter: cols: zu Selektierende Spalten (irrelevant)
#	     tables: Tabellen
#	     where: SQL-WHERE-Bedingung
########################################################################
proc Analyzer::get_anzahl_rows_for_SQL {cols tables where} {
	clear_db_result
	set n [dbcselect $cols $tables $where] ;# Selektion durchfhren

	return $n
}


############### Funktionsbeschreibung ##################################
# Zweck: ermittelt die Anzahl der Antworten fr eine bestimmte Frage
#
# Rckgabewert: Anzahl Antworten fr Frage qst_number
#
# Parameter: formid: Formular-ID
#	     qst_number: Fragenummer mit Prfix qst, z.B. 'qst5'
#
########################################################################
proc Analyzer::get_anzahl_answers_for_question {formid qst_number} {
    global global_options

	set where "WHERE FORMID='$formid' AND QUESTIONNO='$qst_number'"
	return [get_anzahl_rows_for_SQL {"dummy"} $global_options(TBL_RESULTS)\
            $where]
}


############### Funktionsbeschreibung ##################################
# Zweck: ermittelt die Anzahl der nicht-leeren Antworten fr eine Frage
#
# Rckgabewert: Anzahl nicht-leere Antworten fr Frage qst_number
#
# Parameter: formid: Formular-ID
#	     qst_number: Fragenummer mit Prfix qst, z.B. 'qst5'
#
########################################################################
proc Analyzer::get_anzahl_ausgefuellt {formid qst_number} {
    global global_options
	set where "WHERE FORMID='$formid' AND QUESTIONNO='$qst_number'\
        AND ANSWER IS NOT NULL"

	return [get_anzahl_rows_for_SQL {"dummy"} $global_options(TBL_RESULTS)\
        $where]
}


############### Funktionsbeschreibung ##################################
# Zweck: ermittelt die Anzahl der leeren Antworten fr eine Frage
#
# Rckgabewert: Anzahl leere Antworten (NULL) fr Frage qst_number
#
# Parameter: formid: Formular-ID
#	     qst_number: Fragenummer mit Prfix qst, z.B. 'qst5'
#
########################################################################
proc Analyzer::get_anzahl_nicht_ausgefuellt {formid qst_number} {
    global global_options
	set where "WHERE FORMID='$formid' AND QUESTIONNO='$qst_number'\
        AND ANSWER IS NULL"
	return [get_anzahl_rows_for_SQL {"dummy"} $global_options(TBL_RESULTS)\
        $where]
}


############### Funktionsbeschreibung ##################################
# Zweck: gibt den Ausfllstatus fr eine best. Frage zurck
#
# Rckgabewert: Ausfllstatus als xml-Text im Format:
#
#		<ausfuellstatus-section>
#			<ausfuellstatus n=A sum=B nnull=C>
#		</ausfuellstatus-section>
#
#		(mit A=Anzahl nicht-leere Antworten,
#		     B=Anzahl Antworten,
#		     C=Anzahl leere Antworten (NULL) )
#
# Parameter: 
#
########################################################################
proc Analyzer::get_ausfuellstatus_section {formid qst_number} {
	# ermittle Anzahl nicht-leere Antworten:
	set n [get_anzahl_ausgefuellt $formid $qst_number]

	# ermittle Anzahl leere Antworten:
	set n_NULL [get_anzahl_nicht_ausgefuellt $formid $qst_number]

	# xml-Text generieren und zurckgeben
	return "\n<ausfuellstatus-section>\n\t<ausfuellstatus n=\"$n\"\
		sum=\"[expr $n + $n_NULL]\" nnull=\"$n_NULL\">\n\
		</ausfuellstatus-section>"
}


############### Funktionsbeschreibung ##################################
# Zweck: gibt eine xml-Seite zurck, die zur Eingabe eines Delta-Wertes
#	 auffordert
#
# Rckgabewert: Die Seite als fertiger xml-Text im Format
#		<deltavaluepage formid=F passwd=P>
#		(mit F=Frageb.-ID, P=Passwort)
#
# Parameter: formid, password: Fragebogen-ID und Passwort fr den jew.
#	     Fragebogen, werden mitgeschleift, da die erzeugte Seite
#	     wieder auf index.html verweist, diesmal allerdings mit
#	     delta-Wert. formid und password werden in Hidden-Feldern
#	     gespeichert
#
########################################################################
proc Analyzer::get_getDeltaValuePage {formid password} {
	return "<style style=\"analyze.style\">\
		<deltavaluepage formid=\"$formid\"\
		 passwd=\"$password\">"
}


############### Funktionsbeschreibung ##################################
# Zweck: gibt allgemeine Daten zu einem Fragebogen als xml-Text zurck
#
# Rckgabewert: Fragebogendaten als xml-Text im Format
#		<frm-general-data 	frmcreator=A
#					frmcreationdate=B
#					frmdescr=C
#					frmtitle=D
#					frmlastupdate=E>
#		(mit A=Ersteller d. Fragebogens,
#		     B=Erstelldatum d. FB,
#		     C=Beschreibung d. FB,
#		     D=Titel des FB,
#		     E=Letzte nderung d. FB )
#
# Parameter: formid: Fragebogen-ID
#
########################################################################
proc Analyzer::get_general_frm_data {formid} {
	variable ORDERNR_FRM_GENERAL_DATA
	set order_nr $ORDERNR_FRM_GENERAL_DATA

	# Attribute ermitteln:
	set frm_creator [get_value_for_attribute $formid $order_nr "FrmCreatedBy"]
	set frm_creation_date [get_value_for_attribute $formid $order_nr\
        "FrmCreatedOn"]
	set frm_descr [get_value_for_attribute $formid $order_nr "FrmDescription"]
	set frm_title [get_value_for_attribute $formid $order_nr "FrmTitle"]
	set frm_last_update [get_value_for_attribute $formid $order_nr\
        "FrmLastUpdated"]

	return "\n<frm-general-data 	frmcreator=\"$frm_creator\"\
					frmcreationdate=\"$frm_creation_date\"\
					frmdescr=\"$frm_descr\"\
					frmtitle=\"$frm_title\"\
					frmlastupdate=\"$frm_last_update\"\
		>"
}


############### Funktionsbeschreibung ##################################
# Zweck: ermittelt, wie oft eine bestimmte Auswahlmglichkeit
#	 fr eine best. Auswahlfrage gewhlt wurde.
#
# Rckgabewert: Hufigkeit der Option
#
# Parameter: option_code: Code der Option
#		Bsp: option_code=3  (z.B. fr 'Rot')
#		     Antworten in der DB (ANSWER):
#			0 2 3
#			1
#			2 4
#			3
#			0 2
#			==> Option mit Code 3 wurde 2x gewhlt
#	     formid: Fragebogen-ID
#	     qst_number: Nummer der Frage, z.B. 'qst5' oder 'qst3-2'
#
########################################################################
proc Analyzer::get_option_count {option_code formid qst_number} {
    global global_options

	dputs "\nZhle Optionen fr den Wert ---$option_code---, $formid,\
        $qst_number"

	# alle Antworten fr die Frage selektieren
	clear_db_result
	set where "WHERE FORMID='$formid' AND QUESTIONNO='$qst_number'"
	dputs "ANSWERS FOR $qst_number: [dbcselect {ANSWER}\
        $global_options(TBL_RESULTS) $where]"
	dputs "Parsing ANSWERS..."

	# Antworten durchlaufen
	set n 0 ;# abs. Hufigkeit der Auswahlmglichkeit
	set row [dbcfetch]
	while {![is_empty_row $row]} {
		# Antwort-ermitteln:
		set answers [lindex [lindex $row 1] 0]
		dputs "ANSWERS: $answers"

		# wenn der gesuchte Option-Code unter den gegebenen Antworten
		# ist, n erhhen
		if {[expr [lsearch $answers $option_code] > -1] ||\
			(($option_code == "") && ($answers == ""))} {

				incr n
		}

		set row [dbcfetch]
	}
	return $n
}


############### Funktionsbeschreibung ##################################
# Zweck: ermittelt die Optionen fr eine Frage im DB-Rohformat
#        (nur Auswahl-, Blockfrage)
#
# Rckgabewert: Optionen zur Frage bei order_nr im DB-Rohformat,
#               falls diese nicht existieren, {}
#
# Parameter:    formid: Fragebogen-ID
#               order_nr: Orderno. der Frage
#
########################################################################
proc Analyzer::get_options {formid order_nr} {
	return [get_value_for_attribute $formid $order_nr "InpSelOptionValues"]
}


############### Funktionsbeschreibung ##################################
# Zweck: ermittelt die Nummer der Frage bei order_nr
#
# Rckgabewert: Fragenummer, z.B. 'qst3'
#
# Parameter: formid: Fragebogen-ID,
#            order_nr: Orderno. der Frage
#
########################################################################
proc Analyzer::get_qst_number {formid order_nr} {
    global global_options

    # Selektieren des Fragenr-Attributes fr order_nr
    clear_db_result
	set where [subst {where FORMID='$formid' AND ORDERNO=$order_nr\
        AND ATTRIBUTE='QstNumber'}]

	dbcselect {VALUE} $global_options(TBL_FORMDATA) $where
	set res [dbcfetch]

    # vorhanden: Fragenr. mit 'qst' als Prfix zurckgeben
	if {![is_empty_row $res]} {
		set res "qst[lindex  $res 1]"
	} else {
        # Fragenr. nicht vorhanden: {} zurck
		set res {}
	}

	return $res
}


############### Funktionsbeschreibung ##################################
# Zweck: ermittelt zu einer Auswahl- oder Blockfrage, ob es sich
#        um eine Frage mit Einfach- oder Mehrfachauswahl handelt
#
# Rckgabewert: Fragetyp als beschreibender Text
#
# Parameter: formid: Fragebogen-ID,
#            order_nr: Ordernr. der Frage
#
########################################################################
proc Analyzer::get_qst_auswahltyp_as_text {formid order_nr} {
	set einfachauswahl {nur Einfachauswahl}
	set mehrfachauswahl {Mehrfachauswahl mglich}

    # Auswahltyp feststellen und zurckgeben
    if {[is_mehrfachauswahl $formid $order_nr]} {
		return $mehrfachauswahl
	} else {
		return $einfachauswahl
	}
}


############### Funktionsbeschreibung ##################################
# Zweck: ermittelt zu einer Frage allgemeine Fragedaten und gibt diese
#        als xml-Tag <qst-general-data ...> zurck.
#
# Rckgabewert: <qst-general-data
#                   qsttype=A
#                   orderno=B
#                   qstnumber=C
#                   qsttext=D>
#
#               (mit A=Fragetyp als beschr. Text,
#                    B=Reihenfolgenr. der Frage im FB,
#                    C=Nummer der Frage,
#                    D=Text der Frage)
#
# Parameter: formid: Fragebogen-ID,
#            order_nr: Orderno. der Frage
#
########################################################################
proc Analyzer::get_qst_general_data {formid order_nr} {

    # Attribute fr xml-Tag ermitteln
	set qst_type [get_qst_type_as_text $formid $order_nr]
	set qst_number [get_qst_number $formid $order_nr]
	set qst_text [get_qst_text $formid $order_nr]

    # Tag zurckgeben
	return "\n<qst-general-data qsttype=\"$qst_type\" orderno=\"$order_nr\"\
		qstnumber=\"$qst_number\" qsttext=\"$qst_text\">\n"
}


############### Funktionsbeschreibung ##################################
# Zweck: ermittelt den Text einer Frage
#
# Rckgabewert: Fragetext
#
# Parameter: formid: Fragebogen-ID,
#            order_nr: Orderno. der Frage
#
########################################################################
proc Analyzer::get_qst_text {formid order_nr} {
	return [get_value_for_attribute $formid $order_nr "QstText"]
}


############### Funktionsbeschreibung ##################################
# Zweck: ermittelt den Typ einer Frage als Text, einschlielich
#        dem Auswahltyp (Einfach- oder Mehrfachauswahl)
#
# Rckgabewert: Fragetyp als Text
#
# Parameter: formid: Fragebogen-ID,
#            order_nr: Orderno. der Frage
#
########################################################################
proc Analyzer::get_qst_type_as_text {formid order_nr} {

    # Auswahlfrage?
	if {[is_qst_type_auswahl $formid $order_nr]} {
		return "Auswahlfrage / [get_qst_auswahltyp_as_text $formid $order_nr]"
	} else {

    # Blockfrage?
	if {[is_qst_type_block $formid $order_nr]} {
		return "Blockfrage / [get_qst_auswahltyp_as_text $formid $order_nr]"
	} else {

    # Textfrage?
	if {[is_qst_type_text $formid $order_nr]} {
		return "Textfrage"
	} else {

        # Fragetyp unbekannt
		return {(Fragetyp unbekannt)}
	}}}
}


############### Funktionsbeschreibung ##################################
# Zweck: erstellt einen xml-Text, der statistische Informationen
#        ber eine Auswahlfrage enthlt
#
# Rckgabewert: xml-Text im Format:
#
#               <statistic-section>
#                   <statistic-row>
#                       <statistic-heading>
#                           H_1
#                       </statistic-heading>
#                       <statistic-heading>
#                           H_2
#                       </statistic-heading>
#                       ...
#                       <statistic-heading>
#                           H_N
#                       </statistic-heading>
#                   </statistic-row>
#                   <statistic-row>
#                       <statistic-absolute-value>
#                           SA_1
#                       </statistic-absolute-value>
#                       ...
#                       <statistic-absolute-value>
#                           SA_N
#                       </statistic-absolute-value>
#                   </statistic-row>
#                   <statistic-row>
#                       <statistic-relative-value>
#                           SR_1
#                       </statistic-relative-value>
#                       ...
#                       <statistic-relative-value>
#                           SR_N
#                       </statistic-relative-value>
#                   </statistic-row>
#               </statistic-section>
#
#               (mit H_i (i=1..N, N=Anz. Optionen)
#                       =Name der Option Nr. i,
#                    SA_i (i=1..N)=abs. stat. Huf. f. Option i,
#                    SR_i (i=1..N)=rel. stat. Huf f. Option i
#               )
# Parameter: headings: Liste mit den Optionsbezeichnern,
#               z.B. {rot, grn, blau},
#            hlist_absolute: Liste mit den jew. abs. Hufigkeiten,
#               z.B. {5,3,4}
#            hlist_relative: Liste mit den jew. rel. Hufigkeiten,
#
########################################################################
proc Analyzer::get_statistic_info_for_auswahlfrage\
        {headings hlist_absolute hlist_relative} {

    # Verpacken der Optionsbezeichner in ihre xml-Elemente
	set headings [make_xml_elements $headings "<statistic-heading>"\
        "</statistic-heading>"]

    # Verpacken abs. stat. Werte in ihre xml-Elemente
	set hlist_absolute [make_xml_elements $hlist_absolute\
        "<statistic-absolute-value>" "</statistic-absolute-value>"]

    # Verpacken der Optionsbezeichner in ihre xml-Elemente
	set hlist_relative [make_xml_elements $hlist_relative\
        "<statistic-relative-value>" "%</statistic-relative-value>"]

    # alle 3 Listen zusammenfgen
    set xml_stat_list [list $headings $hlist_absolute $hlist_relative]

    # groe Liste (=3 Listen) in xml-Text konvertieren
	return [create_statistic_xml_info_from_list $xml_stat_list]
}


############### Funktionsbeschreibung ##################################
# Zweck: erstellt einen xml-Text, der statistische Informationen
#        ber eine Blockfrage enthlt
#
# Rckgabewert: wie bei get_statistic_info_for_auswahlfrage, die Aus-
#               wahlfrage ist hier aber eine Unterfrage der Blockfrage
#
# Parameter: headings: Optionsbezeichner,
#            subquestions_as_list: Unterfragen als Liste
#            hlists_absolute: Liste der zu den Unterfragen gehrenden
#               Listen mit absoluten Hufigkeiten
#            hlists_relative: Liste der zu den Unterfragen gehrenden
#               Liste mit relativen Hufigkeiten
#            ausfuellstatus_lists: Liste mit Ausfllstatus (pl.) zu
#               den Unterfragen
#
########################################################################
proc Analyzer::get_statistic_info_for_blockfrage\
	{	headings\
		subquestions_as_list\
		hlists_absolute\
		hlists_relative\
		ausfuellstatus_lists
	} {
	set res {}

    # berschrift "Unterfrage" vor Optionsbez. anfgen
    set headings [linsert $headings 0 "Unterfrage:"]
    # Optionsbezeichner verpacken in xml-Elemente
	set headings [make_xml_elements $headings {<statistic-heading>}\
        {</statistic-heading>}]

    # Unterfragen verpacken in xml-Elemente
	set subquestions_as_list [make_xml_elements $subquestions_as_list\
        {<subquestion-text>} {</subquestion-text>}]


    # alle Unterfragen durchlaufen
	set n [llength $subquestions_as_list]
	set i 0

    while {[expr $i < $n]} {

        # die der akt. Unterfrage zugehrigen abs. Hufigkeiten
        # in xml-Elemente verpacken
		set hlists_absolute [lreplace $hlists_absolute $i $i\
					[make_xml_elements [lindex $hlists_absolute $i]\
					{<statistic-absolute-value>}\
					{</statistic-absolute-value>}]\
				    ]

        # die der akt. Unterfrage zugehrigen rel. Hufigkeiten
        # in xml-Elemente verpacken
		set hlists_relative [lreplace $hlists_relative $i $i\
					[make_xml_elements [linsert [lindex $hlists_relative $i]\
                        0 "&nbsp;"]\
					{<statistic-relative-value>}\
                        {%</statistic-relative-value>}]]

        # nchste Unterfrage
		incr i
	}

    # Nun alle Unterfragen durchlaufen, um eine riesige Liste
    # zusammenzustellen, die die Auswertung der ges. Blockfrage beinhaltet
    set i 0
	foreach hlist_abs $hlists_absolute {

        # fr akt. Unterfragenauswertung die Optionsbezeichner anhngen
		lappend res $headings

        # Namen der Unterfrage vor die abs. Hufigkeiten anfgen
		set hlist_abs [concat [lindex $subquestions_as_list $i] $hlist_abs]

        # Name Unterfrage und abs. Hufigkeiten an Resultat anhngen
		lappend res $hlist_abs

        # akt. rel. Hufigkeiten an Resultat anhngen
		lappend res [lindex $hlists_relative $i]

        # akt. Ausfllstatus an Resultat anhngen
		lappend res "<subquestion-ausfuellstatus>\
                        [lindex $ausfuellstatus_lists $i]\
					</subquestion-ausfuellstatus>"

        # die einzelnen Unterfragen optisch trennen:
		lappend res "<td colspan=\"4\"><sep-blockqst></td>"

        # nchste Unterfrage
		incr i
	}

    # die groe Liste in xml-Text bersetzen
    return [create_statistic_xml_info_from_list $res]
}


############### Funktionsbeschreibung ##################################
# Zweck: ermittelt fr eine Auswahlfrage die Optionen, sowie die
#	 absoluten und relativen Hufigkeiten der Optionen
#
# Rckgabewert: Liste im Format:
#               {{H_1 ... H_N} {SA_1 ... SA_N} {SR_1 ... SR_N}}
#               (mit H_i: Optionsbezeichner (Heading) der i-ten Option,
#                    SA_i: abs. stat. Hufigkeit dieser Option,
#                    SR_i: rel. stat. Hufigkeit dieser Option,
#                    i=1..N, N=anz. Auswahloptionen)
#
#
# Parameter: formid: Fragebogen-ID,
#            qst_number: Fragenummer der Auswahlfrage,
#            options: die Auswahloptionen im DB-Rohformat
#
########################################################################
proc Analyzer::get_stat_list_for_auswahlfrage {formid qst_number options} {
	variable OPTION_REGEX

	set res {}
	set hdg_list {}
	set hlist_absolut {}
	set hlist_relativ {}


    # alle Optionen durchlaufen
	while {[regexp $OPTION_REGEX $options match option_code\
		option_name]} {

		dputs "CODE ---$option_code---"
		dputs "VAL ---$option_name---"

        # aktuelle Option ersetzen, mit Rest weiterarbeiten
		regsub $OPTION_REGEX $options "" options

		dputs "Danach: $options"

        # Name d. akt. Option an die Heading-Liste anfgen
		lappend hdg_list $option_name

        # abs. Hufigkeit dieser Option zhlen
		lappend hlist_absolut [get_option_count $option_code $formid\
            $qst_number]
	}

    # erstelle relative Hufigkeiten anhand abs. Hufigkeiten
	set hlist_relativ [absolute2relative $hlist_absolut $formid $qst_number]

    # die Liste im obigen Format zurckgeben
	return [list $hdg_list $hlist_absolut $hlist_relativ]
}


############### Funktionsbeschreibung ##################################
# Zweck: ermittelt die Nummern aller Unterfragen zu einer Blockfrage
#
# Rckgabewert: Liste der Unterfrage-Nummern, jede Nummer hat das
#               Format qstN mit N=0..M-1
#               (mit M=Anzahl der Unterfragen der Blockfrage)
#               Bsp: {qst3-0 qst3-1 qst3-2}
#
# Parameter: : subquestions: die Unterfrage im DB-Rohformat
#
########################################################################
proc Analyzer::get_subquestion_ids {subquestions qst_number} {
    variable ANSWER_INFIX

    # Zhle Unterfragen
    set n [count_rows $subquestions]
    set res {}

    # fr jede Unterfrage die Fragenummer generieren
    set i 0
    while {$i < $n} {
        lappend res "$qst_number$ANSWER_INFIX$i"
        incr i
    }

    return $res
}


############### Funktionsbeschreibung ##################################
# Zweck: ermittelt die Unterfragen einer Blockfrage
#
# Rckgabewert: Unterfragen im DB-Rohformat,
#               {} falls es keine Unterfragen gibt
#
# Parameter: formid: Fragebogen-ID,
#            order_nr: Ordernr. der Blockfrage
#
########################################################################
proc Analyzer::get_subquestions {formid order_nr} {
	return [get_value_for_attribute $formid $order_nr "InpSelBlockValues"]
}


############### Funktionsbeschreibung ##################################
# Zweck: erstellt eine Liste von Unterfragen anhand der Unterfragen
#        im DB-Rohformat
#
# Rckgabewert: Liste aller Unterfragentexte
#
# Parameter: subquestions: Unterfagen im DB-Rohformat
#
########################################################################
proc Analyzer::get_subquestions_as_list {subquestions} {
	set res {}
	set match {}
	set match1 {}
	set SUBQ_REGEX {(^[^\n]*)\n}

	#dputs "SQ: $subquestions"

    # alle Unterfragen durchlaufen
	while {[regexp $SUBQ_REGEX $subquestions match match1]} {
        # erste Unterfrage an Res. anhngen
		lappend res $match1

        # erste Unterfrage aus String lschen
		regsub $SUBQ_REGEX $subquestions "" subquestions
	}

	# Rest anhngen
	lappend res $subquestions

	return $res
}


############### Funktionsbeschreibung ##################################
# Zweck: ermittelt die Summe von Zahlen einer Liste
#
# Rckgabewert: Summe aller Werte
#
# Parameter: werteliste: die Liste mit Zahlen
#
########################################################################
proc Analyzer::get_sum {werteliste} {
	set res 0
	foreach number $werteliste {
		set res [expr $res + $number]
	}

	return $res
}


############### Funktionsbeschreibung ##################################
# Zweck: ermittelt den Wert eines Attributes einer Frage
#        aus der formdata-Tabelle
#
# Rckgabewert: Wert des gewnschten Attributes
#
# Parameter: formid: Fragebogen-ID,
#            order_nr: Ordernr. der Frage
#            attrname: Name des Attributes
#
########################################################################
proc Analyzer::get_value_for_attribute {formid order_nr attrname} {
    global global_options

    # Attributwert selektieren
	set where "WHERE FORMID='$formid' AND ORDERNO=$order_nr\
        AND ATTRIBUTE='$attrname'"

	clear_db_result
	dbcselect {VALUE} $global_options(TBL_FORMDATA) $where
	set row [dbcfetch]

	if {![is_empty_row $row]} {
        # entfernen der Hochkommata wg. HTML
		return [special2html [lindex [lindex $row 1] 0]]
	} else {return {}}
}


############### Funktionsbeschreibung ##################################
# Zweck: prft, ob eine Zeile aus dem DB-Resultat leer ist
#
# Rckgabewert: 0: Zeile ist nicht leer
#               1 sonst
#
# Parameter: row: Die Zeile (mit dbcfetch geholt)
#
########################################################################
proc Analyzer::is_empty_row {row} {
	# mu mind. 1 Datenelement haben
	return [expr [lindex $row 0] < 1]
}


############### Funktionsbeschreibung ##################################
# Zweck: prft, ob eine bestimmte Frage Mehrfachauswahl zult
#
# Rckgabewert: 1: lt Mehrfachauswahl zu
#               0: sonst
#
# Parameter: formid: Fragebogen-ID,
#            order_nr: Ordernr. der Frage
#
########################################################################
proc Analyzer::is_mehrfachauswahl {formid order_nr} {

    # ermitteln des Eingabetyps
	set inp_sel_type [get_value_for_attribute $formid $order_nr "InpSelType"]

    # prfen, ob der jew. Eingabetyp Mehrfachauswahl zult
	switch $inp_sel_type {
		radio {
			return 0
		}

		kombi {
			return 0
		}

		lists {
			return 0
		}

		check {
			return 1
		}

		listm {
			return 1
		}

		default {
			return 0
		}
	}
}


############### Funktionsbeschreibung ##################################
# Zweck: prft, ob eine Frage eine Auswahlfrage ist
#
# Rckgabewert: 1: ist Auswahlfrage,
#               0: sonst
#
# Parameter: formid: Fragebogen-ID,
#            order_nr: Ordernr. der Frage
#
########################################################################
proc Analyzer::is_qst_type_auswahl {formid order_nr} {

    # prfe, ob die fr eine Auswahlfrage bentigten Attribute exisitieren

	if {[attr_exists $formid $order_nr "InpSelType"]\
	&& ![attr_exists $formid $order_nr "InpSelBlockVALUEs"]}\
        {return 1} else {return 0}
}


############### Funktionsbeschreibung ##################################
# Zweck: prft, ob eine Frage eine Blockfrage ist
#
# Rckgabewert: 1: ist Auswahlfrage,
#               0: sonst
#
# Parameter: formid: Fragebogen-ID,
#            order_nr: Ordernr. der Frage
#
########################################################################
proc Analyzer::is_qst_type_block {formid order_nr} {

    # prfe, ob die fr eine Blockfrage bentigten Attribute exisitieren

	if {[attr_exists $formid $order_nr "InpSelType"]\
	&& [attr_exists $formid $order_nr "InpSelBlockVALUEs"]}\
        {return 1} else {return 0}
}


############### Funktionsbeschreibung ##################################
# Zweck: prft, ob eine Frage eine Textfrage ist
#
# Rckgabewert: 1: ist Textfrage,
#               0: sonst
#
# Parameter: formid: Fragebogen-ID,
#            order_nr: Ordernr. der Frage
#
########################################################################
proc Analyzer::is_qst_type_text {formid order_nr} {

    # prfe, ob die fr eine Blockfrage bentigten Attribute exisitieren

	if {[attr_exists $formid $order_nr "InpTxtLength"]\
		&& [attr_exists $formid $order_nr "InpTxtHeight"]\
		&& [attr_exists $formid $order_nr "InpTxtWidth"]}\
            {return 1} else {return 0}
}


############### Funktionsbeschreibung ##################################
# Zweck: konvertiert Elemente einer Liste in xml-Elemente
#
# Rckgabewert: die Liste liste, ob jedem Element wurde textuell opentag
#               vorangestellt und nach dem Element folgt closetag
#
# Parameter: liste: Liste mit Elementen
#            opentag: dieser Text wird Prfix eines jeden Elements
#            closetag: dieser Text wird Suffix eines jeden Elements
#
########################################################################
proc Analyzer::make_xml_elements {liste opentag closetag} {
	set i 0
	set n [llength $liste]

    # Liste durchlaufen
    while {[expr $i < $n]} {

        # das i-te Listenelement ndern
		set liste [lreplace $liste $i $i "$opentag[lindex $liste $i]$closetag"]
		incr i
	}

	return $liste
}

