
namespace eval Builder {
	variable hide_null_default 1
	variable atoms
	# set the atoms-array that allows fast access to all elements, then
	# start building the questionary from this array
	# parameters:	atomlist: list containing the questionary's data
	# 					maxorderno: highest position number for this questionary,
	#						also the number of visible elements
	namespace export build
}


########################################################################
###########################    Oeffentlich    ##########################
########################################################################


proc Builder::build	{	{atomlist {}}
								{maxorderno 0}
							}	{
	variable atoms
	catch {unset atoms}
	foreach {orderno atom} $atomlist {
		lappend atoms($orderno) $atom
	}
	if {[info exists atoms]} {
		set result [execute $maxorderno]
	} else {
		set result ""
	}
	catch {unset atoms}
	return $result
}


########################################################################
##############################   Privat   ##############################
########################################################################


proc Builder::execute { {maxorderno 0} } {
	set result {
		<style style="page.style">
	}

	for {set i 0} {$i <= $maxorderno} {incr i} {
		set molecule "[getMolecule $i]"
		set identifier [lindex [lindex $molecule 0] 0]
		switch -regexp -- $identifier {
			"FRM_.*" { append result "[buildForm $molecule]"}
			"(INP|QST)_.*" { append result "[buildQuestion $molecule]"}
			"SEP_.*" { append result "[buildSeperator $molecule]"}
			"TXT_.*" { append result "[buildText $molecule]"}			
		}
	}		
	
	append result {
		<page-publicformend>
	}
	return $result
}


########################################################################


proc Builder::getMolecule { {orderno 0} } {
	variable atoms
	if	{[catch {set result $atoms($orderno)}]} {
		set result ""
	}
	return $result
}


########################################################################


proc Builder::mapMolecule	{	{molecule {}}
										{map {}}
									}	{
	array set localmap $map
	foreach pair $molecule {
		set name [lindex $pair 0]
		set value [lindex $pair 1]
		if {! [catch {set varname [lindex $localmap($name) 0]}] } {
			upvar 1 "$varname" localvar
			set localvar "$value"
		} 
	}
		
	foreach {key pair} $map {
		set namevalue [lindex $pair 0]
		set varname [lindex $namevalue 0]
		set default [lindex $namevalue 1]
		upvar 1 $varname localvarname
		if {! [info exists localvarname]} {
			set localvarname "$default"
		}
	}
}


########################################################################


proc Builder::getAtom	{	{molecule {}}
									{identifier ""}
								}	{
	if	{	([llength $molecule] < 1)
		||	([string length $identifier] < 1)
	}	{
		return ""
	} else {
		foreach pair $molecule {
			set name [lindex $pair 0]
			if {[string equal $name $identifier]} {
				return $pair
			}
		}
		return ""
	}
}


########################################################################


proc Builder::buildForm { {molecule {}} } {
	set map {
		FRM_TITLE {title ""}
		FRM_SHOW_TITLE {showtitle 1}
		FRM_BG_COLOR {bgcolor "#ffffff"}
		FRM_FG_COLOR {fgcolor "#000000"}
	}
	mapMolecule $molecule $map
	set result [subst {<page-publicformbegin 
		title="$title"
		showtitle="[Util::is_True $showtitle]"
		bgcolor="$bgcolor"
		textcolor="$fgcolor">
	}]
	return $result
}


########################################################################


proc Builder::buildQuestion { {molecule {}} } {
	variable hide_null_default
	set map {
		QST_TEXT text
		QST_NUMBER number
		QST_SHOW_NUMBER shownumber
		QST_ALIGNMENT align
		QST_ANSWER_ALIGNMENT answeralign
		INP_SEL_TYPE type
		INP_SEL_OPTION_VALUES options
		INP_SEL_DEFAULT default
		INP_SEL_ALIGN_TEXT textalign
		INP_SEL_ALIGN_OPTIONS optionalign
		INP_TXT_WIDTH textwidth
		INP_TXT_LENGTH textlength
		INP_TXT_HEIGHT textheight
		INP_SEL_BLOCK_VALUES blockvalues
		INP_SEL_SUB_QST_ALIGNMENT subquestionalign
	}
	
	mapMolecule $molecule $map
	
	if {$hide_null_default
		&& ([string length $default] < 1)
	}	{
		set default "NULL"
	}
	
	if {[Util::is_True $shownumber]} {
		set numbershown "$number. "
	} else {
		set numbershown ""
	}
	set options [split $options "\n"]
	set result {
		<publicform-question number="$number" numbershown="$numbershown"
			align="$align" text="$text">
	}

	if {[string length "$textwidth"] > 0} {
		append result [buildTextQuestion $textheight]
	} elseif {[string length $blockvalues] > 0} {
		append result [buildBlockQuestion $type $subquestionalign $options $blockvalues $number $optionalign $textalign $default]
	} else {
		append result [buildSelectQuestion $type $optionalign]
	}
	
	append result {
		</publicform-question>
	}

	if {[string equal "$answeralign" "aaal"]} {
		set answeralign "left"
	} else {
		set answeralign "right"
	}		
	
	return [subst -nocommands "$result"]
}


########################################################################


proc Builder::buildSeperator { {molecule {}} } {
	set map {
		SEP_TYPE type
		SEP_DATA data
	}
	mapMolecule $molecule $map
	switch -exact -- $type {
		hr { set result [subst {
			<publicform-horizontalrule height="$data">
		}]	}
		img { set result [subst {
				<publicform-image src="$data">
		}] }
		space { set result [subst {
			<publicform-space height="$data">
		}] }
		default { set result [subst {<TR><TD colspan="3">$molecule</TD></TR>}]}
	}
	return "$result"
}


########################################################################


proc Builder::buildText { {molecule {}} } {
	set map {
		TXT_SIZE size
		TXT_ALIGNMENT align
		TXT_TEXT text
		TXT_COLOR color
		TXT_LINK_TARGET url
	}
	mapMolecule $molecule $map
	if {$size == 0} {
		set size "3"
	} elseif {$size > 0} {
		set size "+$size"
	}
	if {[string length "$url"] > 0} {
		set text [subst {<basic-link url="#question_$url">$text</basic-link>}]
	}	
	set result [subst {
		<publicform-text size="$size" align="$align" color="$color">
			$text
		</publicform-text>
	}]
	return $result
}


########################################################################


proc Builder::buildTextQuestion	{	{height 1}
											}	{
	append result {<publicform-answerrow>}
	
	append result {<publicform-answercell align="$answeralign" wrap="nowrap">}
	
	if {$height < 2} {
		append result {<form-text cginame="qst$number" maxlength="$textlength"
			size="$textwidth">
		}
	} else {
		append result {<form-memo cginame="qst$number" cols="$textwidth"
			rows="$textheight">
		}
	}

	append result {
		</publicform-answercell>
	}

	return $result
}


########################################################################


proc Builder::buildBlockQuestion	{	{type "radio"}
												{subqstalign "right"}
												{options {}}
												{blockvalues ""}
												{number 0}
												{optionalign "horizontal"}
												{textalign "behind"}
												{default ""}
											}	{
	
	
	set result {}

	set block [split "$blockvalues" "\n"]

	foreach option $options {
		set currentoption [split $option {=}]
		lappend optionnames [lindex $currentoption 1]
		lappend optionvalues "[lindex $currentoption 0]= "
	}
	
	set result [subst {
			<form-tdtitle items="$optionnames" default="$default">
	}]
	
	for {set i 0} {$i < [llength "$block"]} {incr i} {
		set blockvalue [lindex $block $i]
		set blocktext [lindex [split $blockvalue {=}] 0]
		set tmp {
			<publicform-subquestion number="$number-$i" align="$subqstalign" 
				text="$blocktext">
		}
				
		switch -exact -- $type {
			check {append tmp {<form-tdcheckboxgroup cginame="qst$number-$i" 
				items="$optionvalues" align="$optionalign" textalign="$textalign"
				default="$default" type="block">
			} }
			radio {append tmp {<form-tdradiogroup cginame="qst$number-$i" 
				items="$optionvalues" align="$optionalign" textalign="$textalign"
				default="$default" type="block">
			} }
			default {append tmp {\$molecule}}
		}
		
		
		append result [subst "$tmp"]
	}
	
	
	return $result
}



########################################################################


proc Builder::buildSelectQuestion	{	{type "radio"}
													{optionalign "horizontal"}
												}	{
	if {[string equal "$optionalign" "horizontal"]} {
		append result {<publicform-answerrow>}
	} 
	
	append result {<publicform-answercell wrap="nowrap">} 

	switch -exact -- $type {
		check {append result {<form-tabledcheckboxgroup cginame="qst$number" 
			items="$options" align="$optionalign" textalign="$textalign"
			optionalign="$answeralign" default="$default">
		} }
		kombi {append result {<form-dropdown cginame="qst$number" 
			items="$options" default="$default">
		} }
		lists {append result {<form-listbox cginame="qst$number" 
			items="$options" default="$default">
		} }
		listm {append result {<form-listbox cginame="qst$number" items="$options"
			multiple="1" default="$default">
		} }
		radio {append result {<form-tabledradiogroup cginame="qst$number" 
			items="$options" align="$optionalign" textalign="$textalign" 
			optionalign="$answeralign" default="$default">
		} }
		default {set result {$molecule}}
	}
	
	append result {
		</publicform-answercell>
	}
	
	return $result
}


########################################################################

