
# Module for generating complex error-messages

namespace eval Error {
	# display an error-message
	namespace export display_Msg
	# display an error-message when working on a questinary's element
	namespace export display_Element_Msg
	# display an error-message when a non-existent questionary was selected
	namespace export invalid_Form_Selected
	# display an error-message when the password given is incorrect
	namespace export wrong_Password
	# display an error-message when a new password has not the min. length
	namespace export password_To_Short
	# display an error-message when a new password and it's confirmation are different
	namespace export passwords_Not_Matching
	# display an error-message when a new questionary has no title
	namespace export no_Title_Provided
	# display an error-message when a new questionary has a title, which is already in use
	namespace export title_Already_In_Use
	# display an error-message when the value entered for a color is incorrect
	namespace export wrong_Color_Format
	# display an error-message when new elements could not be inserted into the database
	namespace export db_Insert_Operation_Failed
	# display an error-message when elements could not be deleted from the database
	namespace export db_Delete_Operation_Failed
	# display an error-message when a form could not be copied completely
	namespace export db_Form_Copy_Operation_Failed
	# display an error-message when a submitted questionary had to be declined
	namespace export submission_Not_Accepted
}



########################################################################
###########################    Oeffentlich    ##########################
########################################################################


proc Error::display_Msg	{	{msg {}}
								}	{
	if {[string length "$msg"] == 0} {
		set msg [special2html {Es ist ein nicht nher beschriebener Fehler
			beim Bearbeiten des Fragebogens aufgetreten.}]
	}
	set result {
		<style style="page.style">
		<page-defaultbegin title="Fehler">
		<body-navigation>
			<body-defaultnavigation>
		</body-navigation>
		<body-content>
			<body-paragraph>
	}
	append result "$msg"
	append result {
			</body-paragraph>
			<basic-divider>
			<static-indexlink>
		</body-content>
		<page-defaultend>
	}
}		


########################################################################


proc Error::display_Element_Msg	{	{msg {}}
											}	{
	if {[string length "$msg"] == 0} {
		set msg [special2html {Es ist ein nicht nher beschriebener Fehler
			beim Bearbeiten des Fragebogenelements aufgetreten.}]
	}
	set result [subst {
		<style style="page.style">
		<page-defaultbegin title="Fehler">
		<body-navigation>
			<body-elementnavigation>
		</body-navigation>
		<body-content>
			<body-paragraph>
	}]
	append result "$msg"
	append result [subst {
			</body-paragraph>
			<basic-divider>
			<static-formlink>
		</body-content>
		<page-defaultend>
	}]
}		


########################################################################


proc Error::invalid_Form_Selected {} {
	set errormessage [special2html {
		Der gewhlte Fragebogen ist ungltig oder existiert nicht mehr. 
		Bitte beachten Sie, da der Browser-Cache abgeschaltet sein mu
		und evtl. zwischengeschaltete Proxies deaktiviert oder umgangen 
		werden mssen, damit immer nur die aktuellen Datenbank-Inhalte 
		angezeigt werden.
	}]
	return [display_Msg $errormessage]
}


########################################################################


proc Error::wrong_Password {} {
	set errormessage [special2html {
		Das eingegebene Pawort stimmt nicht. Bitte versuchen Sie
		es erneut.
	}]
	return [display_Msg $errormessage]
}


########################################################################


proc Error::password_To_Short { {refreshlist {}} } {
	set errormessage [subst {
		[special2html {Das eingegebene Pawort ist zu kurz. Bitte whlen 
		Sie ein Pawort mit mindestens sechs Zeichen.}]
		<form-begin>
			<form-parameterrefresh items="{$refreshlist}">
			<form-element label="Pa&szlig;wort:" valign="center">
				<form-password cginame="FRM_PASSWORD_NEW">
			</form-element>
			<form-element label="Pa&szlig;wort wiederholen:" valign="center">
				<form-password cginame="FRM_PASSWORD_CHECK">
			</form-element>
			<form-defaultbuttons>
		<form-end>
	}]
	return [display_Msg $errormessage]
}


########################################################################


proc Error::passwords_Not_Matching { {refreshlist {}} } {
	set errormessage [subst {
		[special2html {Das eingegebene Pawort und die Besttigung stimmen
		nicht berein. Bitte geben Sie diese Information erneut ein.}]
		<form-begin>
			<form-parameterrefresh items="{$refreshlist}">
			<form-element label="Pa&szlig;wort:" valign="center">
				<form-password cginame="FRM_PASSWORD_NEW">
			</form-element>
			<form-element label="Pa&szlig;wort wiederholen:" valign="center">
				<form-password cginame="FRM_PASSWORD_CHECK">
			</form-element>
			<form-defaultbuttons>
		<form-end>
	}]
	return [display_Msg $errormessage]
}


########################################################################


proc Error::no_Title_Provided { {refreshlist {}} } {
	set errormessage [subst {
		[special2html {Es wurde kein Titel angegeben. Bitte geben Sie 
		einen Titel ein, um den Fragebogen spter wieder zu erkennen.}]
		<form-begin>
			<form-parameterrefresh items="{$refreshlist}">
			<form-element label="Titel:" valign="center">
				<form-text cginame="FRM_TITLE">
			</form-element>
			<form-defaultbuttons>
		<form-end>
	}]
	return [display_Msg $errormessage]
}	


########################################################################


proc Error::title_Already_In_Use	{	{refreshlist {}}
												{title {}}
												{newtitlesuggestion {}}
											}	{
	set errormessage [subst {
		Der von Ihnen angegebene Titel &quot;$title&quot; f&uuml;r den 
		Fragebogen wurde bereits von jemand anderem verwandt. Damit Sie 
		den Fragebogen sp&auml;ter wieder erkennen, geben sie bitte einen 
		anderen Titel ein.
		<form-begin>
			<form-parameterrefresh items="{$refreshlist}">
			<form-element label="Titel:" valign="center">
				<form-text cginame="FRM_TITLE" value="$newtitlesuggestion">
			</form-element>
			<form-defaultbuttons>
		<form-end>
	}]
	return [display_Msg $errormessage]
}


########################################################################


proc Error::wrong_Color_Format	{	{refreshlist {}}
												{wrongcolor {}}
												{name "Hintergrund"}
												{cginame "FRM_BG_COLOR"}
											}	{
	set errormessage [subst {
		Die von Ihnen angegebene ${name}farbe $wrongcolor existiert
		nicht oder hat ein falsches Format. Bitte geben Sie die Farbe 
		entweder als Hexadezimal-Wert im Format &quot;#rrggbb&quot; an 
		oder w&auml;hlen Sie einen der folgenden Farbbezeichner: 
		aqua, black, blue, cyan, fuchsia, gray, green, lime, 
		magenta, maroon, navy, olive, purple, red, silver, 
		teal, white, yellow
		<form-begin>
			<form-parameterrefresh items="{$refreshlist}">
			<form-element label="${name}farbe:" valign="center">
				<static-color cginame="$cginame">
			</form-element>
			<form-defaultbuttons>
		<form-end>
	}]
	return [display_Msg $errormessage]
}


########################################################################


proc Error::db_Insert_Operation_Failed {} {
	set errormessage [special2html {
		Ein Datenbankfehler ist aufgetreten. Einige oder alle der neuen
		Datenstze konnten nicht
		in die Datenbank eingefgt werden. Sollte dieser Fehler hufiger
		auftreten, melden Sie dies bitte dem Administrator.
	}]
	return [display_Msg $errormessage]
}


########################################################################


proc Error::db_Delete_Operation_Failed {} {
	set errormessage [special2html {
		Ein Datenbankfehler ist aufgetreten. Einige oder alle der zu
		lschenden Datenstze konnten nicht
		aus der Datenbank entfernt werden. Sollte dieser Fehler hufiger
		auftreten, melden Sie dies bitte dem Administrator.
	}]
	return [display_Msg $errormessage]
}


########################################################################


proc Error::db_Form_Copy_Operation_Failed {} {
	set errormessage [special2html {
		Ein Datenbankfehler ist aufgetreten. Einige oder alle der zu
		kopierenden Datenstze konnten nicht
		ordnungsgem dupliziert werden. Sollte dieser Fehler hufiger
		auftreten, melden Sie dies bitte dem Administrator.
	}]
	return [display_Msg $errormessage]
}


#########################################################################


proc Error::submission_Not_Accepted {} {
	set errormessage [special2html {
		Es tut uns leid, Ihnen mitteilen zu mssen, da Ihre Einsendung 
		leider nicht akzeptiert wurde. Eine ungltige Eingabe ihrerseits 
		oder ein technisches Problem zwang uns leider dazu, den von Ihnen
		bermittelten Fragebogen zu verwerfen. Sollte dieser Fehler 
		hufiger auftreten, melden Sie dies bitte dem Administrator. 
	}]
	return [display_Msg $errormessage]
}


########################################################################
##############################   Privat   ##############################
########################################################################




