# Tabsize 4 --> <= 80 Zeichen/Zeile


# Namensraum fr das Exportmodul

namespace eval ExportExcel {

	# es werden Funktionen aus dem Analyzsemodul wiederverwendet
	source [file join $global_options(BASEDIR) "analyze.tcl"]

	# Plausibilittsprfung importieren
	source [file join $global_options(BASEDIR) "plausibel.tcl"]

	# Exportfunktion exportieren
	namespace export ExportExcel

	# Konstanten fr Fragetypen
	variable BLOCK_QST 0
	variable AUSWAHL_QST 1
	variable TEXT_QST 2

    	variable LEGENDE_MEHRFACHAUSWAHL "\"1=gewhlt 0=nicht gewhlt\";"

	# Feldbegrenzer, die ersetzt werden mssen:
	variable field_delim {"}
	# Ersetzung durch diesen Begrenzer
	variable subst_delim {'}
	

	# Debug-Modus: 0=aus, 1=an
	variable ExportExcel_DEBUGMODE 0
}


########################################################################
###########################    Oeffentlich    ##########################
########################################################################


############### Funktionsbeschreibung ##################################
# Zweck: Exportiert alle Befragungsergebnisse eines Fragebogens in das
#		 Excel-CSV-Format
#
# Rckgabewert: Text im CSV (Comma Seperated Values)-Format:
#				  "f_0_1"; "f_0_2"; ... "f_0_M"
#				  ...
#				  "f_N_1"; "f_N_2"; ... "f_N_M"
#				(mit N=Anzahl ausgefllte Fragebgen,
#				     M=Anzahl Fragen (inkl. der in diesem Modul
#					 zustzl. generierten Fragen bei Mehrfach-
#					 auswahl und Unterfragen, s. Handbuch)
#
#				Wenn deltavalue leer ist, wird eine xml-Seite zurck-
#				gegeben, die zur Eingabe desselben auffordert
#
# Parameter: formid: ID zu exportierender Fragebogen
#			 password: Passwort fr diesen Fragebogen
#			 deltavalue: Delta-Wert fr die Plausibilittsprfung
#
########################################################################
proc ExportExcel::export {formid password deltavalue} {
	# Fragetypen bekannt machen:
	variable AUSWAHL_QST
	variable BLOCK_QST
	variable TEXT_QST

    global global_options

	# DB-Fehlermeldung fr Debug
	global dbcErrorMessage

	set qst_order_numbers {} ;# Order-Nummern der FRAGEelemente des FB
	set qst_meta_infos {} ;# Meta-Informationen ber alle Fragen,
						  ;# spart DB-Zugriffe, wenn diese im Speicher
						  ;# immer parat sind
	set res {} ;# CSV-Resultat



	# Password-Prfung
	set passwd_msg [Util::check_FormID_And_Password $formid $password]

	# Passwort ok...
	if {$passwd_msg == {}} {

	# wenn Delta-Wert leer ist --> entspr. Formular z. Eingabe anzeigen
	if {$deltavalue == {}} {
	    return [Analyzer::get_getDeltaValuePage $formid $password]
	}

	# Plausibilittsprfung durchfhren mit deltavalue
	Plausibel::plausibilitaetspruefung $formid $deltavalue

	# Versuche Datenbank zu ffnen...
	set dbstatus [dbcopen]
	add_debug_msg res "ffne DB (1=OK)... $dbstatus"
	add_debug_msg res "DBC Errormessage <$dbcErrorMessage>"

	add_debug_msg res "Formular: $formid"
	add_debug_msg res "Delta-Wert: $deltavalue Sekunden"
	add_debug_msg res "PW-MSG: $passwd_msg"

	# Datenbank konnte geffnet werden...
	if {$dbstatus} {

		set where "WHERE FORMID='$formid' ORDER BY SESSIONID,QUESTIONNO ASC"
		add_debug_msg res "[dbcselect {*} $global_options(TBL_RESULTS) $where]\
            Antworten gefunden."

		# Hole Anzahl Fragen
		add_debug_msg res  "Ermittle Anzahl der Fragen..."

		set where "WHERE ATTRIBUTE='QstNumber'\
			   AND FORMID='$formid'\
			   ORDER BY ORDERNO ASC"

		Analyzer::clear_db_result
		set qst_count [dbcselect ORDERNO $global_options(TBL_FORMDATA) $where]
		add_debug_msg res  "Anz. Fragen: $qst_count"
		add_debug_msg res  "DBC-Error: $dbcErrorMessage"

		# Prfe Anzahl Fragen...
		if {[expr $qst_count < 1]} {
			# keine Fragen vorhanden --> Fehler
			append res {Der Fragebogen enthlt keine Fragen zum Exportieren!}
		} else {
			add_debug_msg res "Anz. Fragen ok"
			add_debug_msg res "Ermittle Liste der relevanten ORDERNO..."

			# Order-Nummern als Liste ermitteln
			set row [dbcfetch]
			set qst_order_numbers {}
			while {![Analyzer::is_empty_row $row]} {
				lappend qst_order_numbers [lindex $row 1]
				set row [dbcfetch]
			}
		}

		add_debug_msg res "Liste der ORDERNO: $qst_order_numbers"
		add_debug_msg res "\n\n*** Erstellen der Fragen-Meta-Informationen ***"


		# Zusammenstellen der qst_meta_infos, eine Listenstruktur der Form
		# {{qstnumber1} {mehrfachauswahl1} {fragetyp1} {optionen1} {qsttext1}}
		# {{qstnumber2} {mehrfachauswahl2} {fragetyp2} {optionen2} {qsttext2}}
		# ...
		# {{qstnumberN} {mehrfachauswahlN} {fragetypN} {optionenN} {qsttextN}}
		# mit N=Anzahl der Fragen (inkl. der hier zustzl. generierten Fragen)
		# 	  qstnumber: Nummer der Frage, z.B. 'qst5'
		#	  mehrfachauswahl: 0=Einfachauswahl 1=Mehrfachauswahl
		#	  fragetyp: Typ der Frage, s. Konstanten im Namespace-Block oben
		#	  optionen: Auswahloptionen fr die Frage, bei Textfragen leer
		#	  qsttext: Text der Frage bzw. Unterfrage

		# alle Fragen anhand orderno durchlaufen...
		foreach orderno $qst_order_numbers {
			# Fragenummer herausfinden
			set qst_number [Analyzer::get_qst_number $formid $orderno]
			add_debug_msg res "Current qst_no: $qst_number"

			# Ist die Frage mehrfachauswahl?
			set mehrfach [Analyzer::is_mehrfachauswahl $formid $orderno]

			# Auswahloptionen fr die Frage im Rohformat aus der DB
			set options [Analyzer::get_options $formid $orderno]

			# Fragetext (bei Blockfrage Text der allg. Oberfrage)
			set qst_text [Analyzer::get_qst_text $formid $orderno]
            regsub -all -- {\n} $qst_text {} qst_text


			# Auswahlfrage
			if {[Analyzer::is_qst_type_auswahl $formid $orderno]} {
				add_debug_msg res "\tAuswahlfrage!"
				# Frageninformationen anhngen an qst_meta_infos...
				lappend qst_meta_infos [list $qst_number $mehrfach $AUSWAHL_QST\
                    $options $qst_text]
			} else {

			# Blockfrage
			if {[Analyzer::is_qst_type_block $formid $orderno]} {
				add_debug_msg res "\tBlockfrage!"

				# Unterfragen-Nummern herausfinden, z.B. qst3-0, qst3-1 usw.
				set subquestion_ids [Analyzer::get_subquestion_ids\
                    [Analyzer::get_subquestions $formid $orderno] $qst_number]
				set i 0

				# alle Unterfrage durchlaufen
				foreach subquestion_id $subquestion_ids {

					# Text fr Unterfrage ermitteln
					set subquestion_text [get_subquestion_text $formid\
                        $orderno $i]

					# Meta-Info fr diese Unterfrage generieren
					lappend qst_meta_infos [list $subquestion_id $mehrfach\
                        $BLOCK_QST $options $qst_text$subquestion_text]
					incr i
				}
			} else {

			# Textfrage
			if {[Analyzer::is_qst_type_text $formid $orderno]} {
				add_debug_msg res "\tTextfrage!"

				# Frageninformationen an qst_meta_infos anhngen
				lappend qst_meta_infos [list $qst_number 0 $TEXT_QST {}\
                    $qst_text]
			} else {

				# Fragetyp unbekannt --> Fehler
				add_debug_msg res\
                    "\tWeder Auswahl- noch Block- noch Textfrage!: $qst_number"
			}}}
			add_debug_msg res "------------------------------\n\n"
		}

		add_debug_msg res "QST-META-INFOS $qst_meta_infos"



		####################################################################
		# Exportieren ######################################################

		# selektieren der Ergebnisse fr diesen Fragebogen, aufsteigend sor-
		# tiert nach Sitzungs-ID
		Analyzer::clear_db_result
		set where "WHERE FORMID='$formid' ORDER BY SESSIONID, QUESTIONNO ASC"
		add_debug_msg res "[dbcselect {*} $global_options(TBL_RESULTS)\
            $where] answers selected"

		# Kopfzeile fr die CSV-Datei (Fragen)
		append res "[get_excel_first_row $qst_meta_infos]\n"

		# erste Antwort holen
		set row [dbcfetch]

		# curr_results enthlt alle Antworten fr
		# die aktuelle Sitzungs-ID im Format:
		#
		# {{qstnumber1 answer1} ... {qstnumberN answerN}}
		# 	mit N=Anzahl generierte Fragen,
		#		qstnumber=Fragenummer, z.B. 'qst5' oder 'qst3-2'
		#		answer=Antwort auf Frage qstnumber
		#
		# curr_results ist also eine Liste von 2-Tupeln
		set curr_results {}

		# wenn Datenstze vorhanden, die letzte Sitzungs-ID mit der Sitzungs-
		# ID der ersten Antwort initialisieren:
		if {![Analyzer::is_empty_row $row]} {set last_sessionid\
            [lindex [lindex $row 1] 1]}

		# solange noch Antworten vorliegen...
		while {![Analyzer::is_empty_row $row]} {
			add_debug_msg res "********************"

			# es interessiert nur das Antwort-Tupel
			set row [lindex $row 1]

			# wenn die Antworten zu einer neuen Sitzung beginnen...
			# (bzw. wenn eine neue SessionID kommt)
			if {[lindex $row 1] != $last_sessionid} {
				# die Antworten fr die aktuelle Sitzung auswerten
				append res [process_answers $curr_results $qst_meta_infos]

				# die "neue" Sitzungs-ID setzen
				set last_sessionid [lindex $row 1]

				# noch keine Antworten fr die neue Sitzung
				set curr_results {}
			}

			# Die Meta-Information fr diese Frage heraussuchen
			set qst_meta_info [get_qst_meta_info [lindex $row 2]\
                $qst_meta_infos]

			# die Optionen fr diese Frage ermitteln aus Meta-Information
			set options [lindex $qst_meta_info 3]

			# die Fragenummer fr diese Frage ermitteln aus Antwort-Tupel
			set qst_number [lindex $row 2]

			# die Optionen fr diese Frage ermitteln aus Antwort-Tupel
			set answer [lindex $row 3]
            regsub -all -- {\n} $answer {} answer

			# Liste fr aktuelle Antwort an die Antwortenliste fr die
			# aktuelle Sitzung anhngen...
			lappend curr_results [list $qst_number $answer]

			# nchstes Antwort-Tupel einlesen
			set row [dbcfetch]
		}

		# alle Sitzungen durchlaufen, nun noch den "Rest" in
		# curr_results auswerten
		add_debug_msg res "Processing the rest"
		append res [process_answers $curr_results $qst_meta_infos]

		# die letzte CSV-Zeile mit einer Legende zu jeder Frage
		append res "\n\n\n[get_excel_last_row $qst_meta_infos]\n\n"

		# Ungltig-Markierungen der Plausibilitsprfung
		# rckgngig machen
		Plausibel::undo_invalidation $formid

		#Datenbank schlieen
		dbcclose
	} else {
		# Fehler wenn Datenbank nicht geffnet werden konnte
		return "Konnte Datenbank nicht ffnen!"
	}
	} else {
		# Fehler, wenn Passwortberprfung fehlgeschlagen
		return "Fehler bei Passwort-Prfung. Gehen Sie zurck und\
			geben Sie das Passwort erneut ein."
	}


	return $res
}








########################################################################
###########################    Privat    ###############################
########################################################################

############### Funktionsbeschreibung ##################################
# Zweck: Fgt der Variable varname eine Debug-Meldung hinzu, wenn der
#		 DEBUG-Modus an ist
#
# Rckgabewert: -
#
# Parameter: varname: Name der Variable, msg: Debug-Meldung
#
########################################################################
proc ExportExcel::add_debug_msg {varname msg} {
	variable ExportExcel_DEBUGMODE
	upvar $varname text

	if {$ExportExcel_DEBUGMODE} {
		append text "!EXPORT-DEBUG!  $msg<br>"
	}
}


############### Funktionsbeschreibung ##################################
# Zweck: ermittelt aus einer Listenstruktur, die die gleiche Form wie
#		 curr_results in der Hauptfunktion besitzt
#			 ( {{qstnumber1 answer1} {qstnumber2 answerN}} )
#		 die Antwort fr die Frage qst_nr
#
# Rckgabewert: Liste aller Antworten fr die Frage qst_nr
#
# Parameter: qst_nr: Antworten fr diese Frage ermitteln
#			 answers: Antworten im o.g. Format
#
########################################################################
proc ExportExcel::get_answer_list_for_qst {qst_nr answers} {
	set res {}

	# alle Antworten durchlaufen
	foreach answer $answers {
		# wenn Fragenummer gefunden...
		if {[lindex $answer 0] == $qst_nr} {
			return $answer ;# Antwort zurckgeben
		}
	}

	# Antwort fr qst_nr nicht gefunden --> leere Liste
	return $res
}


############### Funktionsbeschreibung ##################################
# Zweck: erstellt aus den qst_meta_infos (Aufbau s. Hauptfunktion)
#		 die Kopfzeile des Excel-Exports, die die Spaltenberschriften
#		 mit allen generierten Fragen beinhaltet.
#
# Rckgabewert: die Kopfzeile im Format:
#				  "F_1"; "F_2"; ... "F_N"
#				  (mit F_i (i=1..N) Name der generierten Frage Nr. i,
#				   N=Anzahl der generierten Fragen)
#
# Parameter: qst_meta_infos: Meta-Informationen ber alle Fragen,
#			 Aufbau s. Hauptfunktion
#
########################################################################
proc ExportExcel::get_excel_first_row {qst_meta_infos} {
	set res {}

	# Meta-Informationen fr jede Frage durchlaufen
	foreach qst_meta_info $qst_meta_infos {

		# wenn Frage mit Mehrfachauswahl: fr jede Option eine neue
		# Frage generieren
		if {[lindex $qst_meta_info 1]} {
			# Optionen ermitteln
			set options [lindex $qst_meta_info 3]
			# Bezeichner der Optionen ermitteln
			set option_names [get_option_names $options]
			# Fragetext ermitteln
			set qst_text [lindex $qst_meta_info 4]
			# Anfhrungszeichen ersetzen, da sonst CSV-Format ersetzt
			foreach option_name $option_names {
				set option_name [replace_Quotes $option_name]
				append res "\"$qst_text (OPTION '$option_name')\";"
			}
		} else {
			# keine Mehrfachauswahl: einfach Fragetext einfgen
			append res "\"[lindex $qst_meta_info 4]\";"
		}
	}

	return $res
}


############### Funktionsbeschreibung ##################################
# Zweck: erstellt aus den qst_meta_infos (Aufbau s. Hauptfunktion)
#		 die Fuzeile des Excel-Exports, die die Legenden
#		 zu allen generierten Fragen beinhaltet.
#
# Rckgabewert: die Fuzeile im Format:
#				  "L_1"; "L_2"; ... "L_N"
#				  (mit L_i (i=1..N) Name der Legende zu Frage Nr. i,
#				   N=Anzahl der generierten Fragen)
#
# Parameter: qst_meta_infos: Meta-Informationen ber alle Fragen,
#			 Aufbau s. Hauptfunktion
#
########################################################################
proc ExportExcel::get_excel_last_row {qst_meta_infos} {
    variable LEGENDE_MEHRFACHAUSWAHL
	set res {}

    # aus Meta-Information Legende fr jede Frage erstellen
	foreach qst_meta_info $qst_meta_infos {

        # Ausw.Optionen fr diese Frage
		set options [lindex $qst_meta_info 3]

        # Anzahl der Optionen fr die Frage ermitteln
		set n [expr 1 +[Analyzer::count_regex "\n" $options]]

        # Anfhrungszeichen in Optionen ersetzen, da sonst
        # CSV-Format verletzt
		set options [replace_Quotes $options]
		regsub -all {\n} $options { / } options

        # wenn Mehrfachauswahl: fr jede Auswahlmglichkeit
        # der akt. Frage eine eigene Frage generieren
		if {[lindex $qst_meta_info 1]} {

			set i 0

            # Standard-Legende fr Auswahlfragen mit Mehrfach-
            # auswahl fr jede generierte Frage
            while {[expr $i < $n]} {
				append res $LEGENDE_MEHRFACHAUSWAHL
				incr i ;# Frage fr nchste Option
			}

		} else {

            # keine Mehrfachauswahl, einfach die Optionen
            # im DB-Rohformat als Legende nehmen
			append res "\"$options\";"
		}
	}

	return $res
}


############### Funktionsbeschreibung ##################################
# Zweck: ermittelt aus den Auswahloptionen einer Auswahlfrage im
#        DB-Rohformat die Codes fr die einzelnen Auswahloptionen als
#        Liste
#
# Rckgabewert: Bsp.:
#                   *** DB:
#                   0=ja\n
#                   1=nein
#
#                   *** Resultat:
#                   {0 1}
#
# Parameter: options: Die Ausw.Optionen im DB-Rohformat
#
########################################################################
proc ExportExcel::get_option_codes {options} {
	set res {}
    set match1 {}
    set match0 {}

	append options \n
	set regex {^([^=]*)=[^\n]*\n}

	while {[regexp $regex $options match0 match1]} {
		regsub $regex $options {} options
		lappend res $match1 ;# Optionscode hinzufgen
	}

	return $res
}


############### Funktionsbeschreibung ##################################
# Zweck: Ermittelt aus Optionen im DB-Rohformat die Namen der Optionen
#
# Rckgabewert: Optionsbezeichner als Liste, z.B. {rot blau gruen}
#
# Parameter: options: Optionen im DB-Rohformat
#
########################################################################
proc ExportExcel::get_option_names {options} {
	set res {}

	append options \n

	set regex {^[^=]*=([^\n]*)\n}

	while {[regexp $regex $options match0 match1]} {
		regsub $regex $options {} options

		lappend res $match1 ;#Optionsname anhngen
	}

	return $res
}


############### Funktionsbeschreibung ##################################
# Zweck: ermittelt die Meta-Information fr die Frage qst_number aus
#        schon bestehenden qst_meta_infos
#
# Rckgabewert: Die Meta-Information fr qst_number (Format s. Haupt-
#               funktion)
#
# Parameter: qst_number: Fr diese Frage (z.B. 'qst5') die Meta-Informa-
#                        tion ermitteln
#            qst_meta_infos: Meta-Informationen ber alle Fragen, Format
#                            s. Hauptfunktion
#
########################################################################
proc ExportExcel::get_qst_meta_info {qst_number qst_meta_infos} {
	set res {}

    # alle Meta-Informationen durchlaufen
    foreach qst_meta_info $qst_meta_infos {

        # wenn Meta-Information zu Frage gefunden...
		if {[lindex $qst_meta_info 0] == $qst_number} {

            # diese zurckgeben
			return $qst_meta_info
		}
	}

	return $res
}


############### Funktionsbeschreibung ##################################
# Zweck: ermittelt den Text fr die i-te Unterfrage einer Blockfrage
#
# Rckgabewert: Text der i-ten Unterfrage, falls diese existiert,
#               sonst eine Fehlermeldung
#
# Parameter: formid: Fragebogen-ID,
#            order_nr: order-Nummer der Frage
#            subqst-nr: Nummer der Unterfrage (Numerierung ab 0..N-1)
#
########################################################################
proc ExportExcel::get_subquestion_text {formid order_nr subqst_nr} {
    global global_options

    # Selektieren des Eintrags mit den Unterfagen
	set where "WHERE FORMID='$formid' AND ORDERNO=$order_nr\
                AND ATTRIBUTE='InpSelBlockValues'"
	dbcselect {VALUE} $global_options(TBL_FORMDATA) $where
	set row [dbcfetch]
	set errormsg {(kann Unterfrage nicht feststellen)}

    # wenn Unterfragen vorhanden...
	if {![Analyzer::is_empty_row $row]} {

        # nur das DB-Tupel interessiert
		set subquestions "[lindex $row 1]\n"

        # Zeilen vor gesuchter Unterfrage ersetzen
		set i 0
		while {[expr $i < $subqst_nr]} {
			regsub {^[^\n]*\n} $subquestions {} subquestions
			incr i
		}

        # alle Zeilen nach gesuchter Zeile entfernen:
        set match0 {}
		set match1 $errormsg
		regexp {^([^\n]*)\n} $subquestions match0 match1

        # bedrohliche Quotes ersetzen
		return "--> [replace_Quotes $match1]"

    } else {

        # keine Unterfagen in DB vorhanden --> Fehlermeldung
    	return $errormsg
	}
}


############### Funktionsbeschreibung ##################################
# Zweck: verarbeitet die Antworten fr einen ausgefllten Fragebogen
#
# Rckgabewert: Excel-CSV-Text mit den Antworten (eine Zeile)
#
# Parameter: curr_results: Die Antworten eines ausgefllten Fragebogens,
#                          Format s. Hauptfunktion
#
########################################################################
proc ExportExcel::process_answers {curr_results qst_meta_infos} {

    # Fragetypen bekannt machen
	variable BLOCK_QST
	variable AUSWAHL_QST
	variable TEXT_QST

	set res {}   ;# das gesamte Resultat (MIT Debug-Meldungen)
	set csv_line {}  ;# Die Excel-CSV-Zeile

	add_debug_msg res "\n *** \n Processing: \nRES:$curr_results \n---\n\
                        META:$qst_meta_infos"

	# Fragen durchlaufen, Reihenfolge der Fragen in der CSV-Zeile ist die
    # Reihenfolge, in der die Fragen in qst_meta_infos stehen
	foreach qst_meta_info $qst_meta_infos {

        # Antworten fr die jew. Frage aus Antworten extrahieren
		set answer [get_answer_list_for_qst [lindex $qst_meta_info 0]\
            $curr_results]

		add_debug_msg res "***\nAnswer: $answer"

        ################################################
        # Blockfrage oder Auswahlfrage gleich behandeln:
		if {([lindex $qst_meta_info 2] == $BLOCK_QST) ||\
            ([lindex $qst_meta_info 2] == $AUSWAHL_QST)} {

            # Optionen ermitteln
			set options [lindex $qst_meta_info 3]
			add_debug_msg res "Options: $options"

            # bei Mehrfachauswahl:
			if {[lindex $qst_meta_info 1]} {
				add_debug_msg res "Mehrfachauswahl (Block/Auswahl)"

                # Codes der Optionen ermitteln
				set option_codes [get_option_codes $options]
				add_debug_msg res "Option Codes: $option_codes"

                # alle Codes durchlaufen und prfen, ob jew. Option
                # gewhlt wurde
				foreach option_code $option_codes {

                    # NULL-Antwort --> nicht gewhlt
					if {[lindex $answer 1] == {}} {
						append csv_line {"0";}
					} else {


					if {[expr [lsearch [lindex $answer 1] $option_code] > -1]} {

                        # Option wurde gewhlt, ist also in der Antwortliste
                        # fr diese Frage vorhanden
						append csv_line {"1";}

					} else {

                        # nein, Option wurde nicht gewhlt
                        append csv_line {"0";}
					}}
				}
			} else {

                # Frage mit Einfachauswahl:
                # einfach Code der Antwort einfgen

				add_debug_msg res "Einfachauswahl"
				append csv_line "\"[lindex $answer 1]\";"
			}

		} else {

        ############
        # Textfrage:

		if {[lindex $qst_meta_info 2] == $TEXT_QST} {
			add_debug_msg res "Textantwort: ---$answer---"

            # Antwort ermitteln und Quotes ersetzen
			set answer [lindex $answer 1]
			set answer [replace_Quotes $answer]

            # Textantwort anfgen
			append csv_line "\"$answer\";"
		}
		}
	}

	add_debug_msg res "\n***\n\n"

    # CSV-Zeile ist neben den Kommentaren Resultat-Bestandteil
    append res "$csv_line\n"

	return $res
}


############### Funktionsbeschreibung ##################################
# Zweck: ersetzt alle Vorkommen von field_delim (s. Namespace-Block)
#        durch subst_delim in s
#
# Rckgabewert: s nach der Ersetzung
#
# Parameter: s: String, in dem ersetzt werden soll
#
########################################################################
proc ExportExcel::replace_Quotes {s} {
	variable field_delim ;# diese Zeichen ersetzen
	variable subst_delim ;# durch diese ersetzen

    # Ersetzung durchfhren
    regsub -all $field_delim $s $subst_delim s

	return $s
}

