
# set the base directory for this tool
set global_options(BASEDIR) "[file dirname [info script]]"

# load external modules
load  [file join "$global_options(BASEDIR)" sql.so]
source [file join "$global_options(BASEDIR)" dbc.tcl]
source [file join "$global_options(BASEDIR)" dbc-mysql.tcl]

# load internal modules
source [file join "$global_options(BASEDIR)" util.tcl]
source [file join "$global_options(BASEDIR)" error.tcl]
source [file join "$global_options(BASEDIR)" cgi.tcl]
source [file join "$global_options(BASEDIR)" form.tcl]
source [file join "$global_options(BASEDIR)" element.tcl]
source [file join "$global_options(BASEDIR)" complexInput.tcl]
source [file join "$global_options(BASEDIR)" dbOperations.tcl]
source [file join "$global_options(BASEDIR)" builder.tcl]
source [file join "$global_options(BASEDIR)" analyze.tcl]
source [file join "$global_options(BASEDIR)" exportExcel.tcl]



namespace eval Generator {
	variable compress_output 0
	variable debug_mode_on 1
	variable debug_collection {}
	variable clock [clock clicks -milliseconds]
	
	namespace export generate_Page
	namespace export generate_XML
	namespace export build_Page
	namespace export debug_Msg
}


########################################################################
###########################    Oeffentlich    ##########################
########################################################################


proc Generator::generate_Page {} {
	set action [CGI::get_Parameter ACTION]
	if  { [DBOperations::db_Is_Open] } {

	switch -exact -- $action {
		el_copy { set result [Element::copy \
			"[CGI::get_Parameter FORMID]" \
			"[CGI::get_Parameter FRM_PASSWORD]" \
			"[CGI::get_Parameter ORDERNO]" \
		] }
		el_delete { set result [Element::delete \
			"[CGI::get_Parameter FORMID]" \
			"[CGI::get_Parameter FRM_PASSWORD]" \
			"[CGI::get_Parameter ORDERNO]" \
		] }
		el_edit { set result [Element::edit \
			"[CGI::get_Parameter FORMID]" \
			"[CGI::get_Parameter FRM_PASSWORD]" \
			"[CGI::get_Parameter ORDERNO]" \
		] }
		el_insert { set result [Element::insert \
			"[CGI::get_Parameter FORMID]" \
			"[CGI::get_Parameter FRM_PASSWORD]" \
			"[CGI::get_Parameter ORDERNO]" \
			"[CGI::get_Parameter TYPE]" \
		] }
		el_append { set result [Element::append_Element \
			"[CGI::get_Parameter FORMID]" \
			"[CGI::get_Parameter FRM_PASSWORD]" \
			"[CGI::get_Parameter TYPE]" \
		] }
		el_move { set result [Element::move \
			"[CGI::get_Parameter FORMID]" \
			"[CGI::get_Parameter FRM_PASSWORD]" \
			"[CGI::get_Parameter ORDERNO]" \
		] }
		el_misc { set result [Element::misc \
			"[CGI::get_Parameter FORMID]" \
			"[CGI::get_Parameter FRM_PASSWORD]" \
		] }
		fbg_copy { set result [Form::copy \
			"[CGI::get_Parameter FORMID]" \
			"[CGI::get_Parameter FRM_PASSWORD]" \
		] }
		fbg_delete { set result [Form::delete \
			"[CGI::get_Parameter FORMID]" \
			"[CGI::get_Parameter FRM_PASSWORD]" \
		] }
		fbg_edit { set result [Form::edit \
			"[CGI::get_Parameter FORMID]" \
			"[CGI::get_Parameter FRM_PASSWORD]" \
		] }
		fbg_generate { set result [DBOperations::fbg_Generate \
			"[CGI::get_Parameter FORMID]" \
		] }
		fbg_new { set result [Form::new] }
		fbg_rename { set result [Form::rename_Form \
			"[CGI::get_Parameter FORMID]" \
			"[CGI::get_Parameter FRM_PASSWORD]" \
		] }
		fbg_eval { set result [Analyzer::analyze \
			"[CGI::get_Parameter FORMID]" \
			"[CGI::get_Parameter FRM_PASSWORD]" \
			"[CGI::get_Parameter LOCKTIME]" \
		] }
		fbg_export { 
			set result [ExportExcel::export \
				"[CGI::get_Parameter FORMID]" \
				"[CGI::get_Parameter FRM_PASSWORD]" \
				"[CGI::get_Parameter LOCKTIME]" \
			] 
			if {[string is integer -strict "[CGI::get_Parameter LOCKTIME]"]} {
				DBOperations::close_DB
				return "$result"
			}
		}
		db_operation { set result [executeDBOperation] }
		index { set result [generateIndex] }
		default { set result [generateIndex] }
	}
		DBOperations::close_DB
	} else {
		set result [Error::display_Msg {
			Konnte die Datenbank nicht &ouml;ffnen! Sollte dieser Fehler 
			h&auml;ufiger auftreten, melden Sie es bitte dem Administrator.
		}]
	}
	return [parseOutput "$result"]
} 


########################################################################


proc Generator::generate_XML	{	{title {[unbenannt]}} 
											{navigation {}}
											{content {}}
										}	{
	if {	([string length "$navigation"] > 0)
		&& ([llength [info procs "$navigation"]] > 0) 
	} {
		if {[catch {set navigation "[$navigation]"} ]} {
			set navigation "<!-- Fehler in Prozedur: $navigation -->"
		}
	} else {
		set navigation "<!-- Unbekannte Prozedur: $navigation -->"
	}
	
	if {	([string length "$content"] > 0)
		&& ([llength [info procs "$content"]] > 0) 
	} {
		if {[catch {set content "[$content]"} ]} {
			set content "<!-- Fehler in Prozedur: $content -->"
		}
	} else {
		set content "<!-- Unbekannte Prozedur: $content -->"
	}

	return [build_Page "$title" "$navigation" "$content"]
}											


########################################################################


proc Generator::build_Page	{	{title {[unbenannt]}} 
										{navigation {}}
										{content {}}
									}	{
	set result {
		<style style="page.style">
		<page-defaultbegin title="$title">
		<body-navigation>$navigation</body-navigation>
		<body-content>$content</body-content>
		<page-defaultend>
	}
	
	return "[subst $result]"
}	


########################################################################


proc Generator::debug_Msg { {text {}} args} {
	variable debug_mode_on
	variable debug_collection
	variable clock
	if {$debug_mode_on} {
		set callingprocedure "[lindex [info level -1] 0]"
		foreach arg $args {
			append text " $arg"
		}
		set message "\[$text\] (from: \[$callingprocedure\] \
			at: \[[expr [clock clicks -milliseconds] - $clock]\])" 
		puts stderr "Fragebogengenerator: $message"
		lappend debug_collection "$message"
	}
}


########################################################################
##############################   Privat   ##############################
########################################################################


proc Generator::generateIndexNavigation {} {
	set result {
		<static-backlink>
	}
	return "$result"
}


########################################################################


proc Generator::generateIndexContent {} {
	set result {}
	append result {
		<form-begin>
			<form-element label="Fragebogen:">
	}
	
	set formlist [DBOperations::build_Form_List]
	if {[llength $formlist] < 28} {
		set size 7
	} else {
		set size [expr "ceil([llength $formlist] / 5)"]
	}
	if {$size < 2} {
		set size 2
	} elseif {$size > 20} {
		set size 20
	}
	append result [ComplexInput::listbox "$formlist" "FORMID" "$size" "100%" "" " "]
	
	append result {
			</form-element>
			<form-element label="Aktion:">
				<static-indexactionlist>
			</form-element>
			<form-element label="Passwort:">
				<static-password>
			</form-element>
			<form-defaultbuttons>
		<form-end>
	}
	return "$result"
}


########################################################################


proc Generator::generateIndex {} {
	return	"[generate_XML \
					Index \
					generateIndexNavigation \
					generateIndexContent \
				]"
}


########################################################################


proc Generator::parseOutput { {text {}} } {
	variable compress_output
	variable debug_mode_on
	variable debug_collection
	debug_Msg "Starte xml2html-Parser..."
	set output [parse_and_trans_html_text "$text"]
	if {$compress_output} {
		regsub -all -- {\s+} "$output" { } output
		regsub -all -- {> <} "$output" {><} output
	}
	debug_Msg "Fertig!"
	if {$debug_mode_on} {
		foreach str $debug_collection {
			append output "<!-- FBG_DEBUG: $str -->\n"
		}
	}
	return [string trim "$output"]
}


########################################################################


proc Generator::executeDBOperation {} {
	set operation [CGI::get_Parameter OPERATION]
		switch -exact -- $operation {
			el_copy { set result [DBOperations::el_Copy \
				"[CGI::get_Parameter FORMID]" \
				"[CGI::get_Parameter FRM_PASSWORD]" \
				"[CGI::get_Parameter ORDERNO]" \
				"[CGI::get_Parameter ORDERNO_NEW]" \
				"[CGI::get_Parameter TYPE]" \
				"[CGI::get_Parameter_List]" \
			] }
			el_delete { set result [DBOperations::el_Delete \
				"[CGI::get_Parameter FORMID]" \
				"[CGI::get_Parameter FRM_PASSWORD]" \
				"[CGI::get_Parameter ORDERNO]" \
			] }
			el_edit { set result [DBOperations::el_Edit \
				"[CGI::get_Parameter FORMID]" \
				"[CGI::get_Parameter FRM_PASSWORD]" \
				"[CGI::get_Parameter ORDERNO]" \
				"[CGI::get_Parameter TYPE]" \
				"[CGI::get_Parameter_List]" \
			] }
			el_insert { set result [DBOperations::el_Insert \
				"[CGI::get_Parameter FORMID]" \
				"[CGI::get_Parameter FRM_PASSWORD]" \
				"[CGI::get_Parameter ORDERNO]" \
				"[CGI::get_Parameter TYPE]" \
				"[CGI::get_Parameter_List]" \
			] }
			el_append { set result [DBOperations::el_Append \
				"[CGI::get_Parameter FORMID]" \
				"[CGI::get_Parameter FRM_PASSWORD]" \
				"[CGI::get_Parameter TYPE]" \
				"[CGI::get_Parameter_List]" \
			] }
			el_move { set result [DBOperations::el_Move \
				"[CGI::get_Parameter FORMID]" \
				"[CGI::get_Parameter FRM_PASSWORD]" \
				"[CGI::get_Parameter ORDERNO]" \
				"[CGI::get_Parameter ORDERNO_NEW]" \
			] }
			fbg_copy { set result [DBOperations::fbg_Copy \
				"[CGI::get_Parameter FORMID]" \
				"[CGI::get_Parameter FRM_TITLE]" \
				"[CGI::get_Parameter FRM_PASSWORD]" \
				"[CGI::get_Parameter FRM_PASSWORD_NEW]" \
				"[CGI::get_Parameter FRM_PASSWORD_CHECK]" \
			]}
			fbg_delete { set result [DBOperations::fbg_Delete \
				"[CGI::get_Parameter FORMID]" \
				"[CGI::get_Parameter FRM_PASSWORD]" \
			] }
			fbg_edit { set result [DBOperations::fbg_Edit \
				"[CGI::get_Parameter FORMID]" \
				"[CGI::get_Parameter FRM_PASSWORD]" \
				"[CGI::get_Parameter FRM_TITLE]" \
				"[CGI::get_Parameter FRM_PASSWORD_NEW]" \
				"[CGI::get_Parameter FRM_PASSWORD_CHECK]" \
				"[CGI::get_Parameter_List]" \
			] }
			fbg_generate { set result [DBOperations::fbg_Generate \
				"[CGI::get_Parameter FORMID]" \
			] }
			fbg_new { set result [DBOperations::fbg_New \
				"[CGI::get_Parameter FRM_TITLE]" \
				"[CGI::get_Parameter FRM_PASSWORD_NEW]" \
				"[CGI::get_Parameter FRM_PASSWORD_CHECK]" \
				"[CGI::get_Parameter_List]" \
			] }
			fbg_rename { set result [DBOperations::fbg_Rename \
				"[CGI::get_Parameter FORMID]" \
				"[CGI::get_Parameter FRM_TITLE]" \
				"[CGI::get_Parameter FRM_PASSWORD]" \
			] }	
			fbg_result { set result [DBOperations::fbg_Result \
				"[CGI::get_Parameter FORMID]" \
				"[CGI::get_Parameter_List]" \
			] }
			default { set result [Error::display_Msg {
					Unbekannte oder unzul&auml;ssige Datenbankoperation!
				}]
			}
		}	
	return "$result"
}


########################################################################

