# Tabsize 4 --> <= 80 Zeichen/Zeile


# Namensraum fr das Plausibilittsprfungs-Modul

namespace eval Plausibel {

    # Hauptfunktion exportieren
	namespace export plausibilitaetspruefung

    # Funktion zum Ent-Markierung ungltiger
    # Fragebgen exportieren
	namespace export undo_invalidation


    # Konstanten:

    # Regexp. fr das SessionID-Format
	variable session_id_fmt\
        {^([0-9]*-[0-9]*-[0-9]*):([0-9]*:[0-9]*):([0-9]*\.[0-9]*)$}

    # Dieses Zeichen wird der FORMID ungltiger
    # Fragebgen voransgestellt. ACHTUNG: Da die Datenbank nur 19 Zeichen
    # fr die FORMID vorsieht, darf das Prfix nur aus einem Zeichen bestehen,
    # da sonst die FORMID zerstrt wird.
	variable INVALID_PREFIX {-}

    # Format einer Zahl
	variable INT_NUMBER_FMT {^[0-9]*$}

    # Default-Delta-Wert
	variable DELTA_DEFAULT 300

    # Debug-Modus, 0=aus, 1=an,
	variable DEBUGMODE 0
}






########################################################################
###########################    Oeffentlich    ##########################
########################################################################

############### Funktionsbeschreibung ##################################
# Zweck: Hauptfunktion fr die Plausibilittsprfung. Markiert nach dem
#        im Programmiererhandbuch beschriebenen Prinzip bestimmte
#        ausgefllte Fragebgen als ungltig, indem der FORMID in der DB
#        ein Prfix vorangestellt wird, welches als Konstante im Name-
#        space-Block festgelegt ist (INVALID_PREFIX).
#
#        Die Markierung der jew. Sitzungsergebnisse mu nach einer
#        entsprechenden Verarbeitung der verbleibenden Datenstze
#        wieder mit der Funktion undo_invalidation rckgngig gemacht
#        werden, damit die sptere Arbeit mit anderen Delta-Werten
#        mglich wird.
#
# Rckgabewert: lediglich ein Text mit Debug-Informationen (sofern
#               Debug-Modus an)
#
# Parameter: formid: Fragebogen-ID des Fragebogens, dessen Antworten
#                    einer Plausibilittsprfung unterzogen werden
#                    sollen
#            delta_sec: Delta-Wert (s. Programmiererhandbuch)
#
########################################################################
proc Plausibel::plausibilitaetspruefung {formid delta_sec} {
	variable INVALID_PREFIX
    global global_options

	set res {}
	set for_removal {}

    # falls Delta-Wert ungltiges Format hat, Default-Wert nehmen
    # (dies ist nur fr den Notfall, eine echte berprfung mit
    #  erneuter Eingabeaufforderung sollte vorher erfolgen!)
	set delta_sec [correct_delta_value $delta_sec]

    # Datenbank ffnen
	if {[dbcopen]} {
		add_debug_msg res "DB is open"

        # ungltige Datenstze entmarkieren
		undo_invalidation $formid

        # alle Session-IDs zu dem Fragebogen FORMID selektieren
		set where "WHERE r.FORMID='$formid' AND r.SESSIONID=l.SESSIONID"
		dbcselect {DISTINCT r.SESSIONID, l.REMOTEHOST}\
            "$global_options(TBL_RESULTS) r, $global_options(RESULTLOG) l"\
            $where


        # alle Session-IDs durchlaufen und in einer Liste speichern
		set row [dbcfetch]
		set session_ids {}
		while {![Analyzer::is_empty_row $row]} {
			set row [lindex $row 1]

			# id / IP
			lappend session_ids [list [lindex $row 0] [lindex $row 1]]

			set row [dbcfetch]
		}

		add_debug_msg res $session_ids


        # jede Session-ID mit jeder anderen Session-ID vergleichen und
        # Zeitdifferenz prfen:
		set n [llength $session_ids]
		set j 0;set k 0
		while {[expr $j < $n]} {
            # Session-ID 0 holen
			set session_info0 [lindex $session_ids $j]

			set k [expr $j + 1]
			while {[expr $k < $n]} {
                # Session-ID 1 holen
				set session_info1 [lindex $session_ids $k]

                # Zeitpunkte t0 und t1 der Session-IDs
				set t0 [lindex $session_info0 0]
				set t1 [lindex $session_info1 0]

                # IP-Adressen der Session-IDs
				set ip0 [lindex $session_info0 1]
				set ip1 [lindex $session_info1 1]

				add_debug_msg res "Vergleiche $t0 und $t1"

                # nur wenn IP-Adressen gleich sind, zeitliche
                # Differenz ermitteln
				if {$ip0 == $ip1} {
                    # Zeitspanne ermitteln
					set diff_sec [get_tdiff_sec $t0 $t1]

					add_debug_msg res "\t--> $diff_sec"

                    # wenn Zeitspanne < Delta-Wert:
					if {[expr $diff_sec < $delta_sec]} {

                        add_debug_msg res "Prfe, ob $t0 in $for_removal..."
                        # wenn Session-ID noch nicht in der liste der
                        # "ungltigen Session-IDs" ist...
						if {[expr [lsearch $for_removal $t0] == -1]} {
                            
                            # ...eine der Session-IDs ist ungltig!
							lappend for_removal $t0
						}
    						add_debug_msg res "\t-->REMOVE!"
					}
				}

				incr k
			}

			incr j
		}

		add_debug_msg res "\nREMOVE: $for_removal"


        # nun alle Fragebgen, die eine ungltige Session-ID haben,
        # markieren
	append res "<red>[llength $for_removal]</red>\
			ungltige Sitzungen mit\
			Delta-Wert=<red>$delta_sec</red>."

        foreach sessionid $for_removal {

			set where "WHERE SESSIONID='$sessionid' AND FORMID='$formid'"

			dbcupdate $global_options(TBL_RESULTS)\
                "FORMID='$INVALID_PREFIX$formid'" $where

		}

		dbcclose  ;# DB schlieen
	} else {
		return {DB Error}  ;# Fehler beim DB-ffnen
	}

	return $res  ;# Debug-Infos zurckgeben
}


############### Funktionsbeschreibung ##################################
# Zweck: Entmarkiert alle als ungltig markierten Datenstze in results-
#        Tabelle
#
# Rckgabewert: -
#
# Parameter: formid: Antworten fr diesen Fragebogen entmarkieren
#
########################################################################
proc Plausibel::undo_invalidation {formid} {
    variable INVALID_PREFIX
    global global_options

    # alle markierten Datenstze selektieren
    set where "WHERE FORMID='$INVALID_PREFIX$formid'"

    # FORMID wiederherstellen
    dbcupdate $global_options(TBL_RESULTS) "FORMID='$formid'" $where
}








########################################################################
###########################    Privat    ###############################
########################################################################


############### Funktionsbeschreibung ##################################
# Zweck: fgt der Variable mit dem Namen varname eine Debug-Meldung hin-
#        zu, sofern der Debug-Modus an ist
#
# Rckgabewert: -
#
# Parameter: varname: Inhalt dieser Variable wird msg hinzugefgt
#
########################################################################
proc Plausibel::add_debug_msg {varname msg} {
	variable DEBUGMODE
	upvar $varname text

	if {$DEBUGMODE} {
		append text "<br>$msg"
	}
}


############### Funktionsbeschreibung ##################################
# Zweck: Korrigiert einen Delta-Wert, falls dieser nicht das Format
#        einer pos. Integer-Zahl hat, in einen Default-Wert (als
#        Konstante im Namespace festgelegt)
#
# Rckgabewert: delta_sec ist pos.Integer-Wert: delta_sec
#               DELTA_DEFAULT sonst
#
# Parameter: delta_sec: ggf. zu korrigierender Delta-Wert
#
########################################################################
proc Plausibel::correct_delta_value {delta_sec} {
    variable INT_NUMBER_FMT
    variable DELTA_DEFAULT

    # wenn delta_sec kein pos. Integer ist...
    if {![regexp $INT_NUMBER_FMT $delta_sec]} {

        # Default-Wert zurckgeben
	    return $DELTA_DEFAULT

    } else {return $delta_sec} ;# ...sonst: keine Korrektur
}


############### Funktionsbeschreibung ##################################
# Zweck: ermittelt die Differenz in Sekunden zwischen zwei Zeitstempeln
#        (wie sie in der DB stehen)
#
# Rckgabewert: Absolute Differenz in Sekunden zwischen den Zeitpunkten,
#               die durch sessionid0 und sessionid1 markiert werden
#
# Parameter: sessionid0, sessionid1: 2 verschiedene Session-IDs,
#            die zwei Fragebogensitzungen identifizieren
#
########################################################################
proc Plausibel::get_tdiff_sec {sessionid0 sessionid1} {
	variable session_id_fmt
	set res 0

    # Default-Werte, falls regulrer Ausdruck nicht pat
	set d0 0
	set t0 0
	set r0 0
	set d1 0
	set t1 0
	set r1 0

    # Datum, Zeit, Rest (Sekundenbruchteile) ermitteln fr
    # beide Session-IDs
	regexp $session_id_fmt $sessionid0 match d0 t0 r0
	regexp $session_id_fmt $sessionid1 match d1 t1 r1

    # Die Session-IDs in Sekunden umwandeln
	set sec0 [expr [clock scan $d0] + [clock scan $t0] + $r0]
	set sec1 [expr [clock scan $d1] + [clock scan $t1] + $r1]

    # Absoluten Wert der zeitl. Differenz ermitteln
    # und zurckgeben
	set res [expr abs([expr $sec1 - $sec0])]

	return $res
}

